///////////////////////////////////////////////////////////////////////////////
// PjRevolutionD Interface class
//
// Last update: 13/01/2008
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Native JAVA imports
///////////////////////////////////////////////////////////////////////////////
import java.awt.Button;
import java.awt.Checkbox;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.Panel;
import java.awt.TextField;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

///////////////////////////////////////////////////////////////////////////////
// JavaView imports
///////////////////////////////////////////////////////////////////////////////
import jv.object.PsPanel;
import jv.objectGui.PsTabPanel;
import jv.object.PsUpdateIf;
import jv.project.PjProject_IP;
import jv.project.PvLightIf;

///////////////////////////////////////////////////////////////////////////////
// Info panel for graph tomography applet
///////////////////////////////////////////////////////////////////////////////
public class PjRevolutionD_IP extends PjProject_IP implements ActionListener, ItemListener
{
   protected PjRevolutionD m_pjRevolutionD;

   protected Panel m_ftPanel; // Transparency panel
   protected Panel m_atPanel; // Transparency panel
   protected Panel m_btPanel; // Transparency panel
   protected Panel m_dtPanel; // Transparency panel
   protected Panel m_aPanel; // Angle panel
   protected Panel m_uPanel; // nuLinesPI panel
   protected Panel m_vPanel; // nvLinesPI panel
   protected Panel m_dPanel; // number_of_disks_PI panel
   
   protected TextField m_fTextField;                    // Text field for f
   protected TextField m_aTextField;                 // Text field for uMin
   protected TextField m_bTextField;                 // Text field for uMax
   
   protected Label msgLabel;   
  
   protected Button updateButton;
   protected Button animateButton;
   
   protected Color errorColor;
   
   protected Checkbox x_cb;
   protected Checkbox y_cb;
   protected Checkbox z_cb;
   protected Checkbox f_cb;
   protected Checkbox a_cb;
   protected Checkbox b_cb;
   protected Checkbox d_cb;
   protected Checkbox i_cb;
   
   protected PsTabPanel tabPanel;

   protected int idiom = 0;
   protected int method = 0;
   protected String[][] title_trans = {{"Usando O Mtodo dos Discos",	
                                        "By The Disk Method"},
                                       {"Usando O Mtodo dos Troncos de Cone",	
                                        "By The Conical Frustrum Method"}};

   protected String[] par_update_trans = {"Atualizar!", "Update!"};
   protected String[] par_animate_trans = {"Animar!", "Animate!"};
   protected String[] par_stop_trans = {"Parar!", "Stop!"};
   protected String[] par_angle_trans = {"Abertura:", "Opening:"};
   protected String[] par_nuLinesPI_trans = {"Discretizao 1:", "Discretization 1:"};  
   protected String[] par_nvLinesPI_trans = {"Discretizao 2:", "Discretization 2:"};
   protected String[][] par_number_of_disks_PI_trans = {{"Discos:", "Disks:"},
		                                                {"Troncos:", "Frusta:"}};
   
   protected String[] cfg_transparency_trans = {"Transparncias",
                                                "Transparencies"};      
   protected String[] cfg_color_trans = {"Cores", "Colors"};
   
   protected String[] cfg_f_transparency_trans = {"f:", "f:"};
   protected String[] cfg_a_transparency_trans = {"a:", "a:"};
   protected String[] cfg_b_transparency_trans = {"b:", "b:"};
   protected String[][] cfg_d_transparency_trans = {{"Discos:", "Disks:"},
		                                            {"Troncos:", "Frusta:"}};
   
   protected String[] cfg_f_pc_trans = {"f: ", "f:"};
   protected String[] cfg_a_pc_trans = {"a: ", "a:"};
   protected String[] cfg_b_pc_trans = {"b: ", "b:"};
   protected String[][] cfg_d_pc_trans = {{"Discos: ", "Disks:"},
		                                  {"Troncos: ", "Frusta:"}};
   
   protected String[] ele_axes_trans = {"Eixos", "Axes"};
   protected String[] ele_edges_trans = {"Malhas", "Malhas"};
   protected String[] ele_lighting_trans = {"Iluminao", "Lighting"};                   
   protected String[] ele_x_cb_trans = {"Exibir eixo x     ", "Show x axis     "};
   protected String[] ele_y_cb_trans = {"Exibir eixo y     ", "Show y axis     "};
   protected String[] ele_z_cb_trans = {"Exibir eixo z", "Show z axis"};
   protected String[] ele_f_cb_trans = {"Exibir malha de f", "Show grid of f"};
   protected String[] ele_a_cb_trans = {"Exibir malha de a", "Show grid of a"};
   protected String[] ele_b_cb_trans = {"Exibir malha de b", "Show grid of b"};
   protected String[][] ele_d_cb_trans = {{"Exibir malha dos discos", "Show grid of disks"},
		                                  {"Exibir malha dos troncos", "Show grid of frusta"}};   
   protected String[] ele_i_cb_trans = {"Habilitar iluminao direcionada (no uniforme)", "Enable lighting direction (non-uniform)"};   

   protected String[] par_panel_trans = {"Parmetros", "Parameters"};   
   protected String[] cfg_panel_trans = {"Configurao", "Configuration"};
   protected String[] ele_panel_trans = {"Elementos", "Elements"};
   
   protected String[] rc_invalid_input_trans = {"Entrada invlida!", "Invalid input!"};
   protected String[] rc_f_eval_trans = {"f no est definida para algum x entre a e b! ", "f is not defined for some x between a and b!"};
   
   protected PsPanel cfgPanel;
   protected PsPanel edgesPanel;
   
   
   ///////////////////////////////////////////////////////////////////////////
   // Default constructor
   ///////////////////////////////////////////////////////////////////////////
   public PjRevolutionD_IP()
   {
      super();

      if (getClass() == PjRevolutionD_IP.class)
      {
         init();
      };
   }; // End of PjRevolutionD_IP()


   //////////////////////////////////////////////////////////////////////////
   // Setup the main control panel
   //////////////////////////////////////////////////////////////////////////
   public void init()
   {
      super.init();

      // Title
      addTitle(title_trans[method][idiom]);

      // add(new Label(" "));
      {
          Label l = new Label(" ");
          l.setFont(new Font("dialog", Font.PLAIN, 2));
          add(l);
      }
                 
      errorColor = new Color(255, 119, 119);

      tabPanel = new PsTabPanel();
      add(tabPanel);
   }; // End of init()


   //////////////////////////////////////////////////////////////////////////
   // Setup parent and add the level slider
   //////////////////////////////////////////////////////////////////////////
   public void setParent(PsUpdateIf parent)
   {
      super.setParent(parent);
      m_pjRevolutionD = (PjRevolutionD) parent;
      idiom = m_pjRevolutionD.idiom;
      method = m_pjRevolutionD.method;
      m_pjRevolutionD.m_pjRevolutionD_IP = this;

      // Parameters panel
      setTitle(title_trans[method][idiom]);
      
      PsPanel parPanel = new PsPanel();
      
      parPanel.add(new Label(" "));           
            
      PsPanel m_mePsPanel = new PsPanel();
      GridBagLayout bl = new GridBagLayout();
      m_mePsPanel.setLayout(bl);      

      Label lf = new Label("f(x) =");
      lf.setFont(new Font("monospaced", Font.PLAIN, 15));
      {
	      GridBagConstraints c = new GridBagConstraints();
	      c.fill = GridBagConstraints.NONE;
	      c.anchor = GridBagConstraints.EAST;
	      c.gridx = 0;
	      c.gridy = 0;
	      c.gridwidth = 1;
	      c.gridheight = 1;
	      c.weightx = 1.0;
	      bl.setConstraints(lf, c);
      }
      m_mePsPanel.add(lf);

      m_fTextField = new TextField();
      m_fTextField.addActionListener(this);
      {
	      GridBagConstraints c = new GridBagConstraints();
	      c.fill = GridBagConstraints.HORIZONTAL;
	      c.gridx = 1;
	      c.gridy = 0;
	      c.gridwidth = 1;
	      c.gridheight = 1;
	      c.weightx = 40.0;
	      bl.setConstraints(m_fTextField, c);
      }
      m_mePsPanel.add(m_fTextField);
                      
      Label la = new Label("a    =");      
      la.setFont(new Font("monospaced", Font.PLAIN, 15));
      {
	      GridBagConstraints c = new GridBagConstraints();
	      c.fill = GridBagConstraints.NONE;
	      c.anchor = GridBagConstraints.EAST;
	      c.gridx = 0;
	      c.gridy = 1;
	      c.gridwidth = 1;
	      c.gridheight = 1;
	      c.weightx = 1.0;
	      bl.setConstraints(la, c);
      }
      m_mePsPanel.add(la);
            
      m_aTextField = new TextField();
      m_aTextField.addActionListener(this);
      {
	      GridBagConstraints c = new GridBagConstraints();
	      c.fill = GridBagConstraints.HORIZONTAL;
	      c.gridx = 1;
	      c.gridy = 1;
	      c.gridwidth = 1;
	      c.gridheight = 1;
	      c.weightx = 20.0;
	      bl.setConstraints(m_aTextField, c);
      }
      m_mePsPanel.add(m_aTextField);
      
      Label lb = new Label("b    =");  
      lb.setFont(new Font("monospaced", Font.PLAIN, 15));
      {
	      GridBagConstraints c = new GridBagConstraints();
	      c.fill = GridBagConstraints.NONE;
	      c.anchor = GridBagConstraints.EAST;
	      c.gridx = 0;
	      c.gridy = 2;
	      c.gridwidth = 1;
	      c.gridheight = 1;
	      c.weightx = 1.0;
	      bl.setConstraints(lb, c);
      }
      m_mePsPanel.add(lb);
      
      m_bTextField = new TextField();
      m_bTextField.addActionListener(this);
      {
	      GridBagConstraints c = new GridBagConstraints();
	      c.fill = GridBagConstraints.HORIZONTAL;
	      c.gridx = 1;
	      c.gridy = 2;
	      c.gridwidth = 1;
	      c.gridheight = 1;
	      c.weightx = 20.0;
	      bl.setConstraints(m_bTextField, c);
      }
      m_mePsPanel.add(m_bTextField);      
      
      parPanel.add(m_mePsPanel);
      
      {
          Label l = new Label(" ");
          l.setFont(new Font("dialog", Font.PLAIN, 1));
          parPanel.add(l);
      }      
            
      msgLabel = new Label(" ", Label.CENTER);
      msgLabel.setForeground(Color.RED);
      msgLabel.setFont(new Font("dialog", Font.PLAIN, 12));
      parPanel.add(msgLabel);                       

      {
          Label l = new Label(" ");
          l.setFont(new Font("dialog", Font.PLAIN, 1));
          parPanel.add(l);
      }            
      
      PsPanel m_bPsPanel = new PsPanel();
      GridBagLayout bp = new GridBagLayout();
      m_bPsPanel.setLayout(bp);
      parPanel.add(m_bPsPanel);
      
      {
    	  Label l = new Label(" ");
	      GridBagConstraints c = new GridBagConstraints();
	      c.fill = GridBagConstraints.HORIZONTAL;	      
	      c.gridx = 0;
	      c.gridy = 0;
	      c.weightx = 1.0;
	      bp.setConstraints(l, c);
	      m_bPsPanel.add(l);
      }            
      
      updateButton = new Button(par_update_trans[idiom]);
      updateButton.addActionListener(this);      
      {
	      GridBagConstraints c = new GridBagConstraints();
	      c.fill = GridBagConstraints.HORIZONTAL;	      
	      c.gridx = 1;
	      c.gridy = 0;
	      c.weightx = 20.0;
	      bp.setConstraints(updateButton, c);
      }
      m_bPsPanel.add(updateButton);

      {
    	  Label l = new Label(" ");
	      GridBagConstraints c = new GridBagConstraints();
	      c.fill = GridBagConstraints.HORIZONTAL;	      
	      c.gridx = 2;
	      c.gridy = 0;
	      c.weightx = 1.0;
	      bp.setConstraints(l, c);
	      m_bPsPanel.add(l);
      }            
      
      animateButton = new Button(par_animate_trans[idiom]);
      animateButton.addActionListener(this);    
      {
	      GridBagConstraints c = new GridBagConstraints();
	      c.fill = GridBagConstraints.HORIZONTAL;
	      c.gridx = 3;
	      c.gridy = 0;
	      c.gridwidth = 1;
	      c.gridheight = 1;
	      c.weightx = 20.0;
	      bp.setConstraints(animateButton, c);
      }
      m_bPsPanel.add(animateButton);
      
      {
    	  Label l = new Label(" ");
	      GridBagConstraints c = new GridBagConstraints();
	      c.fill = GridBagConstraints.HORIZONTAL;	      
	      c.gridx = 4;
	      c.gridy = 0;
	      c.weightx = 1.0;
	      bp.setConstraints(l, c);
	      m_bPsPanel.add(l);
      }            
     
      
      {
          Label l = new Label(" ");
          l.setFont(new Font("dialog", Font.PLAIN, 3));
          parPanel.add(l);
      }     
     
      parPanel.addLine(1);      
      
      m_aPanel = new Panel();
      m_aPanel.setLayout(new GridLayout(1, 1));
      parPanel.add(m_aPanel);
      m_aPanel.removeAll();
      m_pjRevolutionD.angle.setName(par_angle_trans[idiom]);
      m_aPanel.add(m_pjRevolutionD.angle.newInspector(PsPanel.INFO_EXT));
      
      parPanel.addLine(1);
      
      m_uPanel = new Panel();
      m_uPanel.setLayout(new GridLayout(1, 1));
      parPanel.add(m_uPanel);
      m_uPanel.removeAll();
      m_pjRevolutionD.nuLinesPI.setName(par_nuLinesPI_trans[idiom]);
      m_uPanel.add(m_pjRevolutionD.nuLinesPI.newInspector(PsPanel.INFO_EXT));

      m_vPanel = new Panel();
      m_vPanel.setLayout(new GridLayout(1, 1));
      parPanel.add(m_vPanel);
      m_vPanel.removeAll();
      m_pjRevolutionD.nvLinesPI.setName(par_nvLinesPI_trans[idiom]);
      m_vPanel.add(m_pjRevolutionD.nvLinesPI.newInspector(PsPanel.INFO_EXT));
      
      m_dPanel = new Panel();
      m_dPanel.setLayout(new GridLayout(1, 1));
      parPanel.add(m_dPanel);
      m_dPanel.removeAll();
      m_pjRevolutionD.number_of_disks_PI.setName(par_number_of_disks_PI_trans[method][idiom]);
      m_dPanel.add(m_pjRevolutionD.number_of_disks_PI.newInspector(PsPanel.INFO_EXT));
                          
      parPanel.addLine(1);
      
      tabPanel.addPanel(par_panel_trans[idiom], parPanel);      
      
      // Appareance panel
      cfgPanel = new PsPanel();
      
      {
         Label l = new Label(" ");
         l.setFont(new Font("dialog", Font.PLAIN, 6));
         cfgPanel.add(l);
      }
      
      Label transparencyLabel = new Label(cfg_transparency_trans[idiom], Label.CENTER);
      transparencyLabel.setFont(new Font("dialog", Font.BOLD, 13));
      cfgPanel.add(transparencyLabel);   

      {
          Label l = new Label(" ");
          l.setFont(new Font("dialog", Font.PLAIN, 2));
          cfgPanel.add(l);
      }
            
      m_ftPanel = new Panel();
      m_ftPanel.setLayout(new GridLayout(1, 1));
      cfgPanel.add(m_ftPanel);
      m_ftPanel.removeAll();
      m_pjRevolutionD.f_transparency.setName(cfg_f_transparency_trans[idiom]);
      m_ftPanel.add(m_pjRevolutionD.f_transparency.newInspector(PsPanel.INFO_EXT));
      
      m_atPanel = new Panel();
      m_atPanel.setLayout(new GridLayout(1, 1));
      cfgPanel.add(m_atPanel);
      m_atPanel.removeAll();
      m_pjRevolutionD.a_transparency.setName(cfg_a_transparency_trans[idiom]);
      m_atPanel.add(m_pjRevolutionD.a_transparency.newInspector(PsPanel.INFO_EXT));
      
      m_btPanel = new Panel();
      m_btPanel.setLayout(new GridLayout(1, 1));
      cfgPanel.add(m_btPanel);
      m_btPanel.removeAll();
      m_pjRevolutionD.b_transparency.setName(cfg_b_transparency_trans[idiom]);
      m_btPanel.add(m_pjRevolutionD.b_transparency.newInspector(PsPanel.INFO_EXT));
      
      m_dtPanel = new Panel();
      m_dtPanel.setLayout(new GridLayout(1, 1));
      cfgPanel.add(m_dtPanel);
      m_dtPanel.removeAll();
      m_pjRevolutionD.d_transparency.setName(cfg_d_transparency_trans[method][idiom]);
      m_dtPanel.add(m_pjRevolutionD.d_transparency.newInspector(PsPanel.INFO_EXT));
            
      {
          Label l = new Label(" ");
          l.setFont(new Font("dialog", Font.PLAIN, 2));
          cfgPanel.add(l);
      }
      
      cfgPanel.addLine(1);                 
      
      Label colorLabel = new Label(cfg_color_trans[idiom], Label.CENTER);
      colorLabel.setFont(new Font("dialog", Font.BOLD, 13));
      cfgPanel.add(colorLabel);
      
      {
          Label l = new Label(" ");
          l.setFont(new Font("dialog", Font.PLAIN, 2));
          cfgPanel.add(l);
      }      
                            
      m_pjRevolutionD.f_pc_IP.setTitle(cfg_f_pc_trans[idiom]);            
      cfgPanel.add(m_pjRevolutionD.f_pc_IP);
      m_pjRevolutionD.a_pc_IP.setTitle(cfg_a_pc_trans[idiom]);            
      cfgPanel.add(m_pjRevolutionD.a_pc_IP);
      m_pjRevolutionD.b_pc_IP.setTitle(cfg_b_pc_trans[idiom]);            
      cfgPanel.add(m_pjRevolutionD.b_pc_IP);
      m_pjRevolutionD.d_pc_IP.setTitle(cfg_d_pc_trans[method][idiom]);            
      cfgPanel.add(m_pjRevolutionD.d_pc_IP);      
      
      Label lc = new Label(" ");
      lc.setFont(new Font("dialog", Font.PLAIN, 2));
      cfgPanel.add(lc);                            
      
      tabPanel.addPanel(cfg_panel_trans[idiom], cfgPanel);
      
      // Elements panel
      PsPanel elePanel = new PsPanel();
      
      {
          Label l = new Label(" ");
          l.setFont(new Font("dialog", Font.PLAIN, 8));
          elePanel.add(l);
      }
      
      // elePanel.addLine(1);
      
      Label axesLabel = new Label(ele_axes_trans[idiom], Label.CENTER);
      axesLabel.setFont(new Font("dialog", Font.BOLD, 13));
      elePanel.add(axesLabel);
      
      {
          Label l = new Label(" ");
          l.setFont(new Font("dialog", Font.PLAIN, 2));
          elePanel.add(l);
      }            
      
      PsPanel axesPanel = new PsPanel();
      axesPanel.setLayout(new GridLayout(1, 3));
      
      x_cb = new Checkbox(ele_x_cb_trans[idiom]);
      x_cb.addItemListener(this);
      x_cb.setState(m_pjRevolutionD.x_cb_ini);
      axesPanel.add(x_cb);
      
      y_cb = new Checkbox(ele_y_cb_trans[idiom]);
      y_cb.addItemListener(this);
      y_cb.setState(m_pjRevolutionD.y_cb_ini);
      axesPanel.add(y_cb);
      
      z_cb = new Checkbox(ele_z_cb_trans[idiom]);
      z_cb.addItemListener(this);
      z_cb.setState(m_pjRevolutionD.z_cb_ini);
      axesPanel.add(z_cb);
      
      elePanel.add(axesPanel);

      {
          Label l = new Label(" ");
          l.setFont(new Font("dialog", Font.PLAIN, 2));
          elePanel.add(l);
      }
      
      elePanel.addLine(1);
      
      Label edgesLabel = new Label(ele_edges_trans[idiom], Label.CENTER);
      edgesLabel.setFont(new Font("dialog", Font.BOLD, 13));
      elePanel.add(edgesLabel);
      
      {
          Label l = new Label(" ");
          l.setFont(new Font("dialog", Font.PLAIN, 2));
          elePanel.add(l);
      }                  
         
      edgesPanel = new PsPanel();
      edgesPanel.setLayout(new GridLayout(4, 1));
      
      f_cb = new Checkbox(ele_f_cb_trans[idiom]);
      f_cb.addItemListener(this);
      f_cb.setState(m_pjRevolutionD.f_cb_ini);
      edgesPanel.add(f_cb);

      a_cb = new Checkbox(ele_a_cb_trans[idiom]);
      a_cb.addItemListener(this);
      a_cb.setState(m_pjRevolutionD.a_cb_ini);
      edgesPanel.add(a_cb);
      
      b_cb = new Checkbox(ele_b_cb_trans[idiom]);
      b_cb.addItemListener(this);
      b_cb.setState(m_pjRevolutionD.b_cb_ini);
      edgesPanel.add(b_cb);

      d_cb = new Checkbox(ele_d_cb_trans[method][idiom]);
      d_cb.addItemListener(this);
      d_cb.setState(m_pjRevolutionD.d_cb_ini);
      edgesPanel.add(d_cb);      
      
      elePanel.add(edgesPanel);
      
      elePanel.addLine(1);
      
      Label lightingLabel = new Label(ele_lighting_trans[idiom], Label.CENTER);
      lightingLabel.setFont(new Font("dialog", Font.BOLD, 13));
      elePanel.add(lightingLabel);
      
      {
          Label l = new Label(" ");
          l.setFont(new Font("dialog", Font.PLAIN, 2));
          elePanel.add(l);
      }                  
      
      i_cb = new Checkbox(ele_i_cb_trans[idiom]);
      i_cb.addItemListener(this);
      i_cb.setState(m_pjRevolutionD.i_cb_ini);
      elePanel.add(i_cb); 
      
      // elePanel.addLine(1);
      
      tabPanel.addPanel(ele_panel_trans[idiom], elePanel);      
      
      // Validate
      tabPanel.validate();

      super.setParent(parent);
   }; // End of setParent()

   
   //////////////////////////////////////////////////////////////////////////
   // Update method dependent translations
   //////////////////////////////////////////////////////////////////////////   
   public void changeMethod(int idiom, int method)
   {
      setTitle(title_trans[method][idiom]);
      m_pjRevolutionD.d_pc_IP.setTitle(cfg_d_pc_trans[method][idiom]);

      m_dPanel.removeAll();
      m_pjRevolutionD.number_of_disks_PI.setName(par_number_of_disks_PI_trans[method][idiom]);
      m_dPanel.add(m_pjRevolutionD.number_of_disks_PI.newInspector(PsPanel.INFO_EXT));
      m_dPanel.validate();      
      
      m_dtPanel.removeAll();
      m_pjRevolutionD.d_transparency.setName(cfg_d_transparency_trans[method][idiom]);
      m_dtPanel.add(m_pjRevolutionD.d_transparency.newInspector(PsPanel.INFO_EXT));
      m_dtPanel.validate();
      
      this.validate();
      
      d_cb.setLabel(ele_d_cb_trans[method][idiom]);
   }

   //////////////////////////////////////////////////////////////////////////
   // Update the panel whenever the parent has changed somewhere else
   //////////////////////////////////////////////////////////////////////////
   public boolean update(Object event)
   {
      if (m_pjRevolutionD == event)
      {
         PsPanel.setText(m_fTextField, new String(m_pjRevolutionD.f));
         PsPanel.setText(m_aTextField, new String(m_pjRevolutionD.a));
         PsPanel.setText(m_bTextField, new String(m_pjRevolutionD.b));         
         
         return(true);
      };
      
      return(super.update(event));
   }; // End of update()


   //////////////////////////////////////////////////////////////////////////
   // Handle action events invoked from text fields
   //////////////////////////////////////////////////////////////////////////
   public void actionPerformed(ActionEvent event)
   {
      if (m_pjRevolutionD == null)
      {
          return;
      };
         
      Object source = event.getSource();

      if ((source == m_fTextField) || 
          (source == m_aTextField) || (source == m_bTextField) ||
          (source == updateButton))
      {
    	  int rc = 0;
    	  m_pjRevolutionD.f = m_fTextField.getText();
    	  m_pjRevolutionD.a = m_aTextField.getText();
    	  m_pjRevolutionD.b = m_bTextField.getText();
    		  
    	  rc = m_pjRevolutionD.updateSurface();    	  
    	  
    	  if (rc == 0)
    	  {
    		  msgLabel.setText(" ");
    		  m_aTextField.setBackground(Color.WHITE);
    		  m_bTextField.setBackground(Color.WHITE);
    		  m_fTextField.setBackground(Color.WHITE);

    		  m_pjRevolutionD.updateAxes();
    		  
    		  m_pjRevolutionD.updateDisks();
    		  m_pjRevolutionD.disp.setName(m_pjRevolutionD.displayTitle);
    		  
    		  m_pjRevolutionD.disp.fit();
    		  
    		  return;
    	  }
    	  
    	  if (rc == 1)
    	  {
    	     msgLabel.setText(rc_invalid_input_trans[idiom]);
    	     m_aTextField.setBackground(errorColor);
    	     return;
    	  }
    	  
    	  if (rc == 2)
    	  {
     	     msgLabel.setText(rc_invalid_input_trans[idiom]);
    	     m_bTextField.setBackground(errorColor);
    	     return;    		  
    	  }    	  
    	  
    	  if (rc == 3)
    	  {
     	     msgLabel.setText(rc_invalid_input_trans[idiom]);
    	     m_fTextField.setBackground(errorColor);
    	     return;    		  
    	  }    	  
    	  
    	  if (rc == 4)
    	  {
        	 msgLabel.setText(rc_f_eval_trans[idiom]);
        	 m_fTextField.setBackground(errorColor);
        	 return;    		      		      		  
    	  }    	      	  
      } 
      
      if (source == animateButton)
      {
    	  if (animateButton.getLabel().compareTo(par_animate_trans[idiom]) == 0)
    	  {
    		  animateButton.setLabel(par_stop_trans[idiom]);    		 
    		  animateButton.setBackground(new Color(255, 119, 119));
    	  }
    	  else
    	  {
    		  animateButton.setLabel(par_animate_trans[idiom]);
    		  animateButton.setBackground(new Color(212, 208, 200));
    	  }
    	  
    	  m_pjRevolutionD.animate();
      }
      
   }; // End of actionPerformed()


   //////////////////////////////////////////////////////////////////////////
   // Handle action events invoked from itens
   //////////////////////////////////////////////////////////////////////////
   public void itemStateChanged(ItemEvent event)
   {
       if (m_pjRevolutionD == null)
       {
          return;
       };

       Object source = event.getSource();
       
       if (source == x_cb)
       {
    	   if (x_cb.getState() == false)
    	   {
    	      m_pjRevolutionD.removeGeometry(m_pjRevolutionD.axis3);
    	   }
    	   else
    	   {
    		  m_pjRevolutionD.addGeometry(m_pjRevolutionD.axis3);
    	   }
       }
       
       if (source == y_cb)
       {
    	   if (y_cb.getState() == false)
    	   {
    	      m_pjRevolutionD.removeGeometry(m_pjRevolutionD.axis1);
    	   }
    	   else
    	   {
    		  m_pjRevolutionD.addGeometry(m_pjRevolutionD.axis1);
    	   }
       }
       
       if (source == z_cb)
       {
    	   if (z_cb.getState() == false)
    	   {
    	      m_pjRevolutionD.removeGeometry(m_pjRevolutionD.axis2);
    	   }
    	   else
    	   {
    		  m_pjRevolutionD.addGeometry(m_pjRevolutionD.axis2);
    	   }
       }

       if (source == f_cb)
       {
    	   boolean v = f_cb.getState();
    	   m_pjRevolutionD.f_show_edges = v;
           m_pjRevolutionD.ssrf.showEdges(v);
           m_pjRevolutionD.ssrf.update(m_pjRevolutionD.ssrf);
       }

       if (source == a_cb)
       {
    	   boolean v = a_cb.getState();
    	   m_pjRevolutionD.a_show_edges = v;
           m_pjRevolutionD.ssra.showEdges(v);
           m_pjRevolutionD.ssra.update(m_pjRevolutionD.ssra);
       }
       
       if (source == b_cb)
       {
    	   boolean v = b_cb.getState();
    	   m_pjRevolutionD.b_show_edges = v;
           m_pjRevolutionD.ssrb.showEdges(v);
           m_pjRevolutionD.ssrb.update(m_pjRevolutionD.ssrb);
       }
       
       if (source == d_cb)
       {
    	   boolean v = d_cb.getState();
    	   m_pjRevolutionD.d_show_edges = v;
           m_pjRevolutionD.ssrd.showEdges(v);
           m_pjRevolutionD.ssrd.update(m_pjRevolutionD.ssrd);
       }       
       
       if (source == i_cb)
       {
    	   if (i_cb.getState() == false)
    	   {
    	      // m_pjRevolutionD.disp.setLightingModel(PvLightIf.LIGHT_AMBIENT);
    		  m_pjRevolutionD.disp.showDepthcue(false); 
    	      m_pjRevolutionD.disp.update(m_pjRevolutionD.disp);
    	   }
    	   else
    	   {
    		  // m_pjRevolutionD.disp.setLightingModel(PvLightIf.LIGHT_DIRECTION);
    		  m_pjRevolutionD.disp.showDepthcue(true);    		   
    		  m_pjRevolutionD.disp.update(m_pjRevolutionD.disp);
    	   }       
       }
   }; // End of itemStateChanged()
}; // End of PjRevolutionD_IP class
