///////////////////////////////////////////////////////////////////////////////
// PaRevolutionG class
//
// Last update: 23/01/2008
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Native JAVA imports
///////////////////////////////////////////////////////////////////////////////
import java.awt.Color;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.lang.Double;
import java.net.URL;


///////////////////////////////////////////////////////////////////////////////
// JavaView imports
///////////////////////////////////////////////////////////////////////////////
import jv.anim.PsAnimation;
import jv.anim.PsTimeEvent;
import jv.geom.PgPolygon;
import jv.geom.PgPolygonSet;
import jv.number.PdColor;
import jv.number.PuDouble;
import jv.number.PuInteger;
import jv.object.PsPanel;
import jv.project.PjProject;
import jv.project.PvCameraIf;
import jv.project.PvDisplayIf;
import jv.project.PvLightIf;
import jv.vecmath.PdVector;

///////////////////////////////////////////////////////////////////////////////
// PjRevolutionG applet
///////////////////////////////////////////////////////////////////////////////
public class PjRevolutionG extends PjProject 
{
	protected int idiom;
	protected URL codebase;
	protected URL fullpath;
	
    double angle_ini;
    double f_transparency_ini;
    int f_R_ini;
    int f_G_ini; 
    int f_B_ini;
    boolean x_cb_ini;
    boolean y_cb_ini;
    boolean z_cb_ini;    
    boolean f_cb_ini;
    boolean i_cb_ini;	

	protected PuDouble f_transparency;
    protected PuDouble angle;
	
	protected PuInteger nvLinesPI;
	
	protected PvDisplayIf disp;
    protected PvCameraIf cam;
    
    protected KeyListener internalKeyListener;
    
    protected int nvLines = 20;
    
    protected PgPolygon meridianf;
    
    protected SurfaceOfRevolution ssrf;
    
    protected PdColor f_pc;   
    protected PsPanel f_pc_IP;
    
    protected PgPolygonSet axis1;
    protected PgPolygonSet axis2;
    protected PgPolygonSet axis3;
        
    protected boolean f_show_edges = true;
    
    protected PsAnimation ac;               // Animation control
    protected int number_of_frames = 180;
    
    protected double fieldOfView = 0.5;
    
    protected boolean isShowing_f = true;

    
	//////////////////////////////////////////////////////////////////////////
	// Default constructor
	//////////////////////////////////////////////////////////////////////////
	public PjRevolutionG(URL input_codebase, 
			             int input_idiom,
			             int input_nvLines,
			             double input_angle,
			             double input_f_transparency,
			             int input_f_R, int input_f_G, int input_f_B,
			             boolean input_x_cb, boolean input_y_cb, boolean input_z_cb,
			             boolean input_f_cb, 
			             boolean input_i_cb)		
	{
		super("Spatial Geometry Applet");

		codebase = input_codebase;
		idiom = input_idiom;
		nvLines = input_nvLines;
		
	    angle_ini = input_angle;
	    f_transparency_ini = input_f_transparency;
	    f_R_ini = input_f_R;
	    f_G_ini = input_f_G; 
	    f_B_ini = input_f_B;
	    x_cb_ini = input_x_cb;
	    y_cb_ini = input_y_cb;
	    z_cb_ini = input_z_cb;    
	    f_cb_ini = input_f_cb;
	    i_cb_ini = input_i_cb;
	    
	    f_show_edges = f_cb_ini;
	    
	    // Animation control
	    ac = new PsAnimation();
	    ac.setName("Animation");
	    ac.addTimeListener(this);	    

		if (getClass() == PjRevolutionG.class) 
		{
			init();
		};
	}; // End of PjRevolutionG()
	
	//////////////////////////////////////////////////////////////////////////
	// Time event handling
	//////////////////////////////////////////////////////////////////////////
	public boolean setTime(PsTimeEvent te)
	{
	    angle.setValue(te.getTime());
	    
		double a = (360 - angle.getValue())*Math.PI/180.0;
		ssrf.setAngle(a);
		ssrf.update(ssrf);
		
	    return(super.update(this));	    
    }; // End of setTime()

	/////////////////////////////////////////////////////////////////////////
	// Animation callback function
	/////////////////////////////////////////////////////////////////////////
	public void animate()
	{
	   if (super.hasAnimation())
	   {		   
	      PsAnimation pa = super.getAnimation();
	      if (pa.isRunning() == true)
	      {
	         pa.stop();  
	      }
	      else
	      {
	         pa.setTimeInterval(0.0, 360.0, 360/number_of_frames, 720/number_of_frames);
	         pa.setTime(angle.getValue());
	         pa.getAnimationPanel().setVisible(false);
	         pa.setRepeat(PsAnimation.BACK_FORTH);
	         pa.start();
	      };
	   };
	}; // End of animate()
	 
	 
	//////////////////////////////////////////////////////////////////////////
	// Initialize variables
	//////////////////////////////////////////////////////////////////////////
	public void init() 
	{
		// Transparency sliders
		f_transparency = new PuDouble("f:", this);
		f_transparency.setDefBounds(0.0, 1.0, 0.01, 0.05);
		f_transparency.setDefValue(f_transparency_ini);
		f_transparency.init();
		f_transparency.setEnabled(true);
		f_transparency.setValue(f_transparency_ini);
		f_transparency.setEnabledConfigButton(false);
						
		// Angle
		angle = new PuDouble("Abertura:", this);
		angle.setDefBounds(0.0, 360.0, 1, 5);
		angle.setDefValue(angle_ini);
		angle.init();
		angle.setEnabled(true);
		angle.setValue(0.0);
		angle.setEnabledConfigButton(false);
				
		// nvLinesPI
		nvLinesPI = new PuInteger("Discretizao 2:", this);
		nvLinesPI.setDefBounds(3, 100, 1, 4);
		nvLinesPI.setDefValue(nvLines);
		nvLinesPI.init();
		nvLinesPI.setEnabled(true);
		nvLinesPI.setValue(nvLines);
		nvLinesPI.setEnabledConfigButton(false);
		
		// Meridians
	    meridianf = new PgPolygon(3);
	    meridianf.setNumVertices(2);
	    meridianf.setVertex(0, 0, 1, -3);
	    meridianf.setVertex(1, 0, 1, +3);
	    
	    // Surfaces of Revolution f
	    // Color fc = new Color(200, 255, 0);
	    ssrf = new SurfaceOfRevolution(3);
	    ssrf.showTransparency(true);
	    ssrf.setTransparency(f_transparency_ini);
	    ssrf.setMeridian(meridianf);
	    ssrf.setDiscr(nvLines);	    
	    ssrf.setGlobalElementColor(new Color(f_R_ini, f_G_ini, f_B_ini));
	    ssrf.showEdges(f_show_edges);	
	    ssrf.compute();
	    
	    // Colors
	    f_pc = new PdColor("f:", this);
	    f_pc.setColor(new Color(f_R_ini, f_G_ini, f_B_ini));
	    f_pc_IP = new PsPanel();	   
	    f_pc_IP = f_pc.newInspector(PsPanel.INFO_EXT);
	    	    
        // Axes
	    axis1 = new PgPolygonSet(3); // This is a 3D object
        axis1.setNumVertices(2);
        axis1.setNumPolygons(1);
        axis1.getVertex(0).setName("");
        axis1.getVertex(1).setName("x");
        axis1.showVertexLabels(true);
        axis1.setPolygon(0, 0, 1);
        axis1.setGlobalPolygonColor(Color.BLUE);
        axis1.setGlobalVertexColor(Color.BLUE);
        axis1.setPolygonSize(0, 0.5);
        axis1.showPolygonSizes(true);
        axis1.showPolygonEndArrow(true);
        axis1.update(axis1);
        
	    axis2 = new PgPolygonSet(3); // This is a 3D object
        axis2.setNumVertices(2);
        axis2.setNumPolygons(1);
        axis2.getVertex(0).setName("");
        axis2.getVertex(1).setName("y");
        axis2.showVertexLabels(true);
        axis2.setPolygon(0, 0, 1);
        axis2.setGlobalPolygonColor(Color.BLUE);
        axis2.setGlobalVertexColor(Color.BLUE);
        axis2.setPolygonSize(0, 0.5);
        axis2.showPolygonSizes(true);
        axis2.showPolygonEndArrow(true);
        axis2.update(axis2);

	    axis3 = new PgPolygonSet(3); // This is a 3D object
        axis3.setNumVertices(2);
        axis3.setNumPolygons(1);
        axis3.getVertex(0).setName("");
        axis3.getVertex(1).setName("z");
        axis3.showVertexLabels(true);
        axis3.setPolygon(0, 0, 1);
        axis3.setGlobalPolygonColor(Color.BLUE);
        axis3.setGlobalVertexColor(Color.BLUE);
        axis3.setPolygonSize(0, 0.5);
        axis3.showPolygonSizes(true);
        axis3.showPolygonEndArrow(true);
        axis3.update(axis3);
        
        updateAxes();
	}; // End of init()

	//////////////////////////////////////////////////////////////////////////
	// Update point
	//////////////////////////////////////////////////////////////////////////	
	public void updatePoint(String C, double y, double z)
	{
		String s = C;
        s = s.replaceAll("P", "");
        int i = Integer.valueOf(s).intValue();
		meridianf.setVertex(i, 0, y, z);
		ssrf.compute();
		ssrf.update(ssrf);
	}; // End of updatePoint();	
	
	//////////////////////////////////////////////////////////////////////////
	// Update number of points
	//////////////////////////////////////////////////////////////////////////		
	public void updateNumberOfPoints(String C)
	{
	    String s = C;
	    int n_new = Integer.valueOf(s).intValue();

	    // Build new list of points
	    double zStp = 6.0/(n_new - 1);
	    meridianf.setNumVertices(n_new);
	    meridianf.setVertex(0, 0, 1, -3);

	    for (int i = 1; i < n_new; i++)
	    {
	       double zVal = -3.0 + i*zStp;
	       meridianf.setVertex(i, 0, 1, zVal);
	    }
	    
		ssrf.compute();
		ssrf.update(ssrf);	    
	}
	
	//////////////////////////////////////////////////////////////////////////
	// Update axes
	//////////////////////////////////////////////////////////////////////////	
	public void updateAxes()
	{
        double xMin = -3.2;
        double yMin = -3.2;
        double zMin = -5.2;
        double xMax = 3.2;
        double yMax = 3.2;
        double zMax = 5.2;        
		
		// Axes
	    axis1.setVertex(0, new PdVector(xMin, 0, 0));
	    axis1.setVertex(1, new PdVector(xMax, 0, 0));
        axis1.getVertex(0).setName("");
        axis1.getVertex(1).setName("x");
        axis1.showVertexLabels(true);
        axis1.setPolygon(0, 0, 1);
        axis1.setGlobalPolygonColor(Color.BLUE);
        axis1.setGlobalVertexColor(Color.BLUE);
        axis1.setGlobalVertexSize(0.0);
        axis1.setPolygonSize(0, 0.5);
        axis1.showPolygonSizes(true);
        axis1.showPolygonEndArrow(true);
        axis1.update(axis1);
        
	    axis2.setVertex(0, new PdVector(0, yMin, 0));
	    axis2.setVertex(1, new PdVector(0, yMax, 0));
        axis2.getVertex(0).setName("");
        axis2.getVertex(1).setName("y");
        axis2.showVertexLabels(true);
        axis2.setPolygon(0, 0, 1);
        axis2.setGlobalPolygonColor(Color.BLUE);
        axis2.setGlobalVertexColor(Color.BLUE);
        axis2.setGlobalVertexSize(0.0);
        axis2.setPolygonSize(0, 0.5);
        axis2.showPolygonSizes(true);
        axis2.showPolygonEndArrow(true);
        axis2.update(axis2);		

	    axis3.setVertex(0, new PdVector(0, 0, zMin));
	    axis3.setVertex(1, new PdVector(0, 0, zMax));
        axis3.getVertex(0).setName("");
        axis3.getVertex(1).setName("z");
        axis3.showVertexLabels(true);
        axis3.setPolygon(0, 0, 1);
        axis3.setGlobalPolygonColor(Color.BLUE);
        axis3.setGlobalVertexColor(Color.BLUE);
        axis3.setGlobalVertexSize(0.0);
        axis3.setPolygonSize(0, 0.5);
        axis3.showPolygonSizes(true);
        axis3.showPolygonEndArrow(true);
        axis3.update(axis3);		        
	}
	
	
	//////////////////////////////////////////////////////////////////////////
	// Things to do when the class is started
	//////////////////////////////////////////////////////////////////////////
	public void start() 
	{
		super.start();
		
		addGeometry(ssrf);
		if (x_cb_ini == true)
		{
		   addGeometry(axis3);
		}
		if (y_cb_ini == true)
		{
		   addGeometry(axis1);
		}
		if (z_cb_ini == true)
		{
		   addGeometry(axis2);
		}
		update(this);

		disp = getDisplay(); // The PwUnfold needs a reference to the
		                     // display in order to work properly.

		PdVector pos = new PdVector(0, 1, 0);
		PdVector vdi = new PdVector(0, 1, 0);
		cam = disp.getCamera();
		cam.setPosition(pos);
		cam.setViewDir(vdi);
		cam.setRoll(0);
		cam.update(cam);
		disp.fit();		
		
		if (i_cb_ini == false)
		{
  	       // disp.setLightingModel(PvLightIf.LIGHT_AMBIENT);
		   disp.showDepthcue(false);
	       disp.update(disp);			
		}
		else
		{
	  	   // disp.setLightingModel(PvLightIf.LIGHT_DIRECTION);
		   disp.showDepthcue(true);
		   disp.update(disp);						
		}
		
        // Key events
        internalKeyListener = new KeyAdapter() 
        {
            public void keyPressed(KeyEvent keyEvent) 
            {
           	   if (keyEvent.getKeyCode() == KeyEvent.VK_1)
        	   {
           	      if (isShowing_f == true)
           		  {
           			isShowing_f = false;
           			disp.removeGeometry(ssrf);
           			disp.update(disp);
           		  }
           		  else
           		  {
             		isShowing_f = true;
               		disp.addGeometry(ssrf);  
               		disp.update(disp);
           		  }
        	   };
        	   
            	
         	   if (keyEvent.getKeyCode() == KeyEvent.VK_8)
        	   {
        	 	  fieldOfView -= 0.05;
         		  if (fieldOfView <= 0.0)
        		  {
        			  fieldOfView = 0.0;
        		  };
        		  cam.setFieldOfView(fieldOfView);
        		  cam.update(cam);
        	   };        	  
        	  
        	   if (keyEvent.getKeyCode() == KeyEvent.VK_9)
        	   {
        	      fieldOfView += 0.05;
        		  if (fieldOfView >= 2.0)
        		  {
        			  fieldOfView = 2.0;
        		  };
        		  cam.setFieldOfView(fieldOfView);
        		  cam.update(cam);
        	   };        	            	   
            };
        };
        disp.addKeyListener(internalKeyListener);		
	}; // End of start()
	

	/////////////////////////////////////////////////////////////////////////
	// Update the class whenever a child has changed. Method is usually
	// invoked from the children.
	/////////////////////////////////////////////////////////////////////////
	public boolean update(Object event) 
	{
		if (event == null)
		{
			return (true);
		};

		// Solid transparency
		if (event == f_transparency) 
		{			
			ssrf.setTransparency(f_transparency.getValue());
			ssrf.update(ssrf);
			
			return (true);
		};
		
		
		if (event == angle) 
		{			
			double a = (360 - angle.getValue())*Math.PI/180.0;
			ssrf.setAngle(a);
			ssrf.update(ssrf);
			
			return (true);
		};
		
		if (event == nvLinesPI) 
		{			
			nvLines = nvLinesPI.getValue();
			ssrf.setDiscr(nvLines);
			ssrf.update(ssrf);
			
			return (true);
		};
						
		if (event == f_pc)
		{		
			ssrf.setGlobalElementColor(f_pc.getColor());
			ssrf.update(ssrf);
			
			return (true);			
		}
				
		return (super.update(event));
	}; // End of update()
}; // End of PjRevolutionG classs
