///////////////////////////////////////////////////////////////////////////////
// PjRevolutionG Interface class
//
// Last update: 13/01/2008
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Native JAVA imports
///////////////////////////////////////////////////////////////////////////////
import java.awt.Button;
import java.awt.Checkbox;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.Panel;
import java.awt.TextField;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

///////////////////////////////////////////////////////////////////////////////
// JavaView imports
///////////////////////////////////////////////////////////////////////////////
import jv.object.PsPanel;
import jv.objectGui.PsTabPanel;
import jv.object.PsUpdateIf;
import jv.project.PjProject_IP;
import jv.project.PvLightIf;

///////////////////////////////////////////////////////////////////////////////
// Info panel for graph tomography applet
///////////////////////////////////////////////////////////////////////////////
public class PjRevolutionG_IP extends PjProject_IP implements ActionListener, ItemListener
{
   protected PjRevolutionG m_pjRevolutionG;

   protected Panel m_ftPanel; // Transparency panel
   
   protected Panel m_aPanel; // Angle panel
   protected Panel m_vPanel; // nvLinesPI panel
   
   protected Button animateButton;
   
   protected Checkbox x_cb;
   protected Checkbox y_cb;
   protected Checkbox z_cb;
   protected Checkbox f_cb;
   protected Checkbox i_cb;
   
   protected PsTabPanel tabPanel;

   protected int idiom = 0;
   protected int method = 0;
   protected String[] par_animate_trans = {"Animar!", "Animate!"};
   protected String[] par_stop_trans = {"Parar!", "Stop!"};
   protected String[] par_angle_trans = {"Abertura:", "Opening:"};  
   protected String[] par_nvLinesPI_trans = {"Discretizao:", "Discretization:"};
   
   protected String[] cfg_f_transparency_trans = {"Transparncia:", "Transparency:"};
   
   protected String[] cfg_f_pc_trans = {"Cor: ", "Color:"};
   
   protected String[] ele_axes_trans = {"Eixos", "Axes"};
   protected String[] ele_edges_trans = {"Malhas", "Malhas"};
   protected String[] ele_lighting_trans = {"Iluminao", "Lighting"};                   
   protected String[] ele_x_cb_trans = {"Exibir eixo x     ", "Show x axis     "};
   protected String[] ele_y_cb_trans = {"Exibir eixo y     ", "Show y axis     "};
   protected String[] ele_z_cb_trans = {"Exibir eixo z", "Show z axis"};
   protected String[] ele_f_cb_trans = {"Exibir malha da superfcie", "Show surface grid"};
   protected String[] ele_i_cb_trans = {"Habilitar iluminao direcionada (no uniforme)", "Enable lighting direction (non-uniform)"};   

   protected String[] par_panel_trans = {"Parmetros", "Parameters"};   
   protected String[] cfg_panel_trans = {"Configurao", "Configuration"};
   protected String[] ele_panel_trans = {"Elementos", "Elements"};
   
   protected PsPanel cfgPanel;
   protected PsPanel edgesPanel;
   
   
   ///////////////////////////////////////////////////////////////////////////
   // Default constructor
   ///////////////////////////////////////////////////////////////////////////
   public PjRevolutionG_IP()
   {
      super();

      if (getClass() == PjRevolutionG_IP.class)
      {
         init();
      };
   }; // End of PjRevolutionG_IP()


   //////////////////////////////////////////////////////////////////////////
   // Setup the main control panel
   //////////////////////////////////////////////////////////////////////////
   public void init()
   {
      super.init();

      tabPanel = new PsTabPanel();
      add(tabPanel);
   }; // End of init()


   //////////////////////////////////////////////////////////////////////////
   // Setup parent and add the level slider
   //////////////////////////////////////////////////////////////////////////
   public void setParent(PsUpdateIf parent)
   {
      super.setParent(parent);
      m_pjRevolutionG = (PjRevolutionG) parent;
      idiom = m_pjRevolutionG.idiom;

      // Parameters panel
      PsPanel parPanel = new PsPanel();
      
      {
          Label l = new Label(" ");
          l.setFont(new Font("dialog", Font.PLAIN, 6));
          parPanel.add(l);
      }
            
      PsPanel m_bPsPanel = new PsPanel();
      GridBagLayout bp = new GridBagLayout();
      m_bPsPanel.setLayout(bp);
      parPanel.add(m_bPsPanel);
      
      {
    	  Label l = new Label(" ");
	      GridBagConstraints c = new GridBagConstraints();
	      c.fill = GridBagConstraints.HORIZONTAL;	      
	      c.gridx = 0;
	      c.gridy = 0;
	      c.weightx = 1.0;
	      bp.setConstraints(l, c);
	      m_bPsPanel.add(l);
      }            
      
      
      animateButton = new Button(par_animate_trans[idiom]);
      animateButton.addActionListener(this);    
      {
	      GridBagConstraints c = new GridBagConstraints();
	      c.fill = GridBagConstraints.HORIZONTAL;
	      c.gridx = 1;
	      c.gridy = 0;
	      c.gridwidth = 1;
	      c.gridheight = 1;
	      c.weightx = 20.0;
	      bp.setConstraints(animateButton, c);
      }
      m_bPsPanel.add(animateButton);
      
      {
    	  Label l = new Label(" ");
	      GridBagConstraints c = new GridBagConstraints();
	      c.fill = GridBagConstraints.HORIZONTAL;	      
	      c.gridx = 2;
	      c.gridy = 0;
	      c.weightx = 1.0;
	      bp.setConstraints(l, c);
	      m_bPsPanel.add(l);
      }            
     
      
      {
          Label l = new Label(" ");
          l.setFont(new Font("dialog", Font.PLAIN, 3));
          parPanel.add(l);
      }     
     
      
      m_aPanel = new Panel();
      m_aPanel.setLayout(new GridLayout(1, 1));
      parPanel.add(m_aPanel);
      m_aPanel.removeAll();
      m_pjRevolutionG.angle.setName(par_angle_trans[idiom]);
      m_aPanel.add(m_pjRevolutionG.angle.newInspector(PsPanel.INFO_EXT));
      
      m_vPanel = new Panel();
      m_vPanel.setLayout(new GridLayout(1, 1));
      parPanel.add(m_vPanel);
      m_vPanel.removeAll();
      m_pjRevolutionG.nvLinesPI.setName(par_nvLinesPI_trans[idiom]);
      m_vPanel.add(m_pjRevolutionG.nvLinesPI.newInspector(PsPanel.INFO_EXT));
                               
      tabPanel.addPanel(par_panel_trans[idiom], parPanel);      
      
      // Appareance panel
      cfgPanel = new PsPanel();
      
      {
         Label l = new Label(" ");
         l.setFont(new Font("dialog", Font.PLAIN, 6));
         cfgPanel.add(l);
      }

      {
          Label l = new Label(" ");
          l.setFont(new Font("dialog", Font.PLAIN, 2));
          cfgPanel.add(l);
      }      
      
      m_ftPanel = new Panel();
      m_ftPanel.setLayout(new GridLayout(1, 1));
      cfgPanel.add(m_ftPanel);
      m_ftPanel.removeAll();
      m_pjRevolutionG.f_transparency.setName(cfg_f_transparency_trans[idiom]);
      m_ftPanel.add(m_pjRevolutionG.f_transparency.newInspector(PsPanel.INFO_EXT));
      
            
      {
          Label l = new Label(" ");
          l.setFont(new Font("dialog", Font.PLAIN, 2));
          cfgPanel.add(l);
      }
                             
      m_pjRevolutionG.f_pc_IP.setTitle(cfg_f_pc_trans[idiom]);            
      cfgPanel.add(m_pjRevolutionG.f_pc_IP);

      tabPanel.addPanel(cfg_panel_trans[idiom], cfgPanel);
      
      // Elements panel
      PsPanel elePanel = new PsPanel();
      
      {
          Label l = new Label(" ");
          l.setFont(new Font("dialog", Font.PLAIN, 6));
          elePanel.add(l);
      }
      
      PsPanel axesPanel = new PsPanel();
      axesPanel.setLayout(new GridLayout(1, 3));
      
      x_cb = new Checkbox(ele_x_cb_trans[idiom]);
      x_cb.addItemListener(this);
      x_cb.setState(m_pjRevolutionG.x_cb_ini);
      axesPanel.add(x_cb);
      
      y_cb = new Checkbox(ele_y_cb_trans[idiom]);
      y_cb.addItemListener(this);
      y_cb.setState(m_pjRevolutionG.y_cb_ini);
      axesPanel.add(y_cb);
      
      z_cb = new Checkbox(ele_z_cb_trans[idiom]);
      z_cb.addItemListener(this);
      z_cb.setState(m_pjRevolutionG.z_cb_ini);
      axesPanel.add(z_cb);
      
      elePanel.add(axesPanel);

      {
          Label l = new Label(" ");
          l.setFont(new Font("dialog", Font.PLAIN, 2));
          elePanel.add(l);
      }                                      
      
      edgesPanel = new PsPanel();
      edgesPanel.setLayout(new GridLayout(1, 1));
      
      f_cb = new Checkbox(ele_f_cb_trans[idiom]);
      f_cb.addItemListener(this);
      f_cb.setState(m_pjRevolutionG.f_cb_ini);
      edgesPanel.add(f_cb);
      
      elePanel.add(edgesPanel);

      {
          Label l = new Label(" ");
          l.setFont(new Font("dialog", Font.PLAIN, 2));
          elePanel.add(l);
      }                                
      
      i_cb = new Checkbox(ele_i_cb_trans[idiom]);
      i_cb.addItemListener(this);
      i_cb.setState(m_pjRevolutionG.i_cb_ini);
      elePanel.add(i_cb); 
      
      // elePanel.addLine(1);
      
      tabPanel.addPanel(ele_panel_trans[idiom], elePanel);      
      
      // Validate
      tabPanel.validate();

      super.setParent(parent);
   }; // End of setParent()

   
   //////////////////////////////////////////////////////////////////////////
   // Update the panel whenever the parent has changed somewhere else
   //////////////////////////////////////////////////////////////////////////
   public boolean update(Object event)
   {
      //
	   
      return(super.update(event));
   }; // End of update()


   //////////////////////////////////////////////////////////////////////////
   // Handle action events invoked from text fields
   //////////////////////////////////////////////////////////////////////////
   public void actionPerformed(ActionEvent event)
   {
      if (m_pjRevolutionG == null)
      {
          return;
      };
         
      Object source = event.getSource();
      
      if (source == animateButton)
      {
    	  if (animateButton.getLabel().compareTo(par_animate_trans[idiom]) == 0)
    	  {
    		  animateButton.setLabel(par_stop_trans[idiom]);    		 
    		  animateButton.setBackground(new Color(255, 119, 119));
    	  }
    	  else
    	  {
    		  animateButton.setLabel(par_animate_trans[idiom]);
    		  animateButton.setBackground(new Color(212, 208, 200));
    	  }
    	  
    	  m_pjRevolutionG.animate();
      }
      
   }; // End of actionPerformed()


   //////////////////////////////////////////////////////////////////////////
   // Handle action events invoked from itens
   //////////////////////////////////////////////////////////////////////////
   public void itemStateChanged(ItemEvent event)
   {
       if (m_pjRevolutionG == null)
       {
          return;
       };

       Object source = event.getSource();
       
       if (source == x_cb)
       {
    	   if (x_cb.getState() == false)
    	   {
    	      m_pjRevolutionG.removeGeometry(m_pjRevolutionG.axis3);
    	   }
    	   else
    	   {
    		  m_pjRevolutionG.addGeometry(m_pjRevolutionG.axis3);
    	   }
       }
       
       if (source == y_cb)
       {
    	   if (y_cb.getState() == false)
    	   {
    	      m_pjRevolutionG.removeGeometry(m_pjRevolutionG.axis1);
    	   }
    	   else
    	   {
    		  m_pjRevolutionG.addGeometry(m_pjRevolutionG.axis1);
    	   }
       }
       
       if (source == z_cb)
       {
    	   if (z_cb.getState() == false)
    	   {
    	      m_pjRevolutionG.removeGeometry(m_pjRevolutionG.axis2);
    	   }
    	   else
    	   {
    		  m_pjRevolutionG.addGeometry(m_pjRevolutionG.axis2);
    	   }
       }

       if (source == f_cb)
       {
    	   boolean v = f_cb.getState();
    	   m_pjRevolutionG.f_show_edges = v;
           m_pjRevolutionG.ssrf.showEdges(v);
           m_pjRevolutionG.ssrf.update(m_pjRevolutionG.ssrf);
       }

       
       if (source == i_cb)
       {
    	   if (i_cb.getState() == false)
    	   {
    	      // m_pjRevolutionG.disp.setLightingModel(PvLightIf.LIGHT_AMBIENT);
    		  m_pjRevolutionG.disp.showDepthcue(false); 
    	      m_pjRevolutionG.disp.update(m_pjRevolutionG.disp);
    	   }
    	   else
    	   {
    		  // m_pjRevolutionG.disp.setLightingModel(PvLightIf.LIGHT_DIRECTION);
    		  m_pjRevolutionG.disp.showDepthcue(true);    		   
    		  m_pjRevolutionG.disp.update(m_pjRevolutionG.disp);
    	   }       
       }
   }; // End of itemStateChanged()
}; // End of PjRevolutionG_IP class
