///////////////////////////////////////////////////////////////////////////////
// PaRevolutionX class
//
// Last update: 23/01/2008
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Native JAVA imports
///////////////////////////////////////////////////////////////////////////////
import java.awt.Color;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.lang.Double;
import java.net.URL;


///////////////////////////////////////////////////////////////////////////////
// JavaView imports
///////////////////////////////////////////////////////////////////////////////
import jv.anim.PsAnimation;
import jv.anim.PsTimeEvent;
import jv.geom.PgPolygon;
import jv.geom.PgPolygonSet;
import jv.number.PdColor;
import jv.number.PuDouble;
import jv.number.PuInteger;
import jv.object.PsPanel;
import jv.project.PjProject;
import jv.project.PvCameraIf;
import jv.project.PvDisplayIf;
import jv.project.PvLightIf;
import jv.vecmath.PdVector;

///////////////////////////////////////////////////////////////////////////////
// Parser imports
///////////////////////////////////////////////////////////////////////////////
import org.nfunk.jep.JEP;

///////////////////////////////////////////////////////////////////////////////
// PjRevolutionX applet
///////////////////////////////////////////////////////////////////////////////
public class PjRevolutionX extends PjProject 
{
	protected int idiom;
	protected URL codebase;
	protected URL fullpath;
	
    double angle_ini;
    double f_transparency_ini;
    double g_transparency_ini;
    double a_transparency_ini;
    double b_transparency_ini;
    int f_R_ini;
    int f_G_ini; 
    int f_B_ini;
    int g_R_ini;
    int g_G_ini;
    int g_B_ini;
    int a_R_ini;
    int a_G_ini;
    int a_B_ini;
    int b_R_ini;
    int b_G_ini;
    int b_B_ini;    
    boolean x_cb_ini;
    boolean y_cb_ini;
    boolean z_cb_ini;    
    boolean f_cb_ini;
    boolean g_cb_ini;
    boolean a_cb_ini;
    boolean b_cb_ini;    
    boolean i_cb_ini;	

	protected PuDouble f_transparency;
	protected PuDouble g_transparency;
	protected PuDouble a_transparency;
	protected PuDouble b_transparency;	
    protected PuDouble angle;
	
	protected PuInteger nuLinesPI;
	protected PuInteger nvLinesPI;

	protected PvDisplayIf disp;
    protected PvCameraIf cam;
    
    protected KeyListener internalKeyListener;
    
    protected String f;
    protected String g;
    protected String a;
    protected String b;
        
    protected int nuLines = 20;
    protected int nvLines = 20;
    
    protected PgPolygon meridianf;
    protected PgPolygon meridiang;
    protected PgPolygon meridiana;
    protected PgPolygon meridianb;
    
    protected SurfaceOfRevolution ssrf;
    protected SurfaceOfRevolution ssrg;
    protected SurfaceOfRevolution ssra;
    protected SurfaceOfRevolution ssrb;
    
    protected JEP myParserF;
    protected JEP myParserN;
    
    protected double xMin;
    protected double xStp;
    protected double xMax;
    
    protected PdColor f_pc;
    protected PdColor g_pc;
    protected PdColor a_pc;
    protected PdColor b_pc;
    protected PsPanel f_pc_IP;
    protected PsPanel g_pc_IP;
    protected PsPanel a_pc_IP;
    protected PsPanel b_pc_IP;
    
    protected PgPolygonSet axis1;
    protected PgPolygonSet axis2;
    protected PgPolygonSet axis3;
        
    protected boolean f_show_edges = true;
    protected boolean g_show_edges = true;
    protected boolean a_show_edges = true;
    protected boolean b_show_edges = true;    
    
    protected PsAnimation ac;               // Animation control
    protected int number_of_frames = 180;
    
    protected double fieldOfView = 0.5;
    
    protected boolean isShowing_f = true;
    protected boolean isShowing_g = true;
    protected boolean isShowing_a = true;
    protected boolean isShowing_b = true;
    
    
	//////////////////////////////////////////////////////////////////////////
	// Default constructor
	//////////////////////////////////////////////////////////////////////////
	public PjRevolutionX(URL input_codebase, 
			             int input_idiom,
			             String input_f,
			             String input_g,
			             String input_a,
			             String input_b,
			             int input_nuLines,
			             int input_nvLines,
			             double input_angle,
			             double input_f_transparency,
			             double input_g_transparency,
			             double input_a_transparency,
			             double input_b_transparency,
			             int input_f_R, int input_f_G, int input_f_B,
			             int input_g_R, int input_g_G, int input_g_B,
			             int input_a_R, int input_a_G, int input_a_B,
			             int input_b_R, int input_b_G, int input_b_B,
			             boolean input_x_cb, boolean input_y_cb, boolean input_z_cb,
			             boolean input_f_cb, boolean input_g_cb, boolean input_a_cb, boolean input_b_cb,
			             boolean input_i_cb)		
	{
		super("Spatial Geometry Applet");

		codebase = input_codebase;
		idiom = input_idiom;
		f = input_f;
		g = input_g;
		a = input_a;
		b = input_b;
		nuLines = input_nuLines;
		nvLines = input_nvLines;
		
	    angle_ini = input_angle;
	    f_transparency_ini = input_f_transparency;
	    g_transparency_ini = input_g_transparency;
	    a_transparency_ini = input_a_transparency;
	    b_transparency_ini = input_b_transparency;
	    f_R_ini = input_f_R;
	    f_G_ini = input_f_G; 
	    f_B_ini = input_f_B;
	    g_R_ini = input_g_R;
	    g_G_ini = input_g_G;
	    g_B_ini = input_g_B;
	    a_R_ini = input_a_R;
	    a_G_ini = input_a_G;
	    a_B_ini = input_a_B;
	    b_R_ini = input_b_R;
	    b_G_ini = input_b_G;
	    b_B_ini = input_b_B;    
	    x_cb_ini = input_x_cb;
	    y_cb_ini = input_y_cb;
	    z_cb_ini = input_z_cb;    
	    f_cb_ini = input_f_cb;
	    g_cb_ini = input_g_cb;
	    a_cb_ini = input_a_cb;
	    b_cb_ini = input_b_cb;    
	    i_cb_ini = input_i_cb;
	    
	    f_show_edges = f_cb_ini;
	    g_show_edges = g_cb_ini;
	    a_show_edges = a_cb_ini;
	    b_show_edges = b_cb_ini;
	    
	    // Animation control
	    ac = new PsAnimation();
	    ac.setName("Animation");
	    ac.addTimeListener(this);	    

		if (getClass() == PjRevolutionX.class) 
		{
			init();
		};
	}; // End of PjRevolutionX()
	
	//////////////////////////////////////////////////////////////////////////
	// Time event handling
	//////////////////////////////////////////////////////////////////////////
	public boolean setTime(PsTimeEvent te)
	{
	    angle.setValue(te.getTime());
	    
		double a = (360 - angle.getValue())*Math.PI/180.0;
		ssrf.setAngle(a);
		ssrg.setAngle(a);
		ssra.setAngle(a);
		ssrb.setAngle(a);
		ssrf.update(ssrf);
		ssrg.update(ssrg);
		ssra.update(ssra);
		ssrb.update(ssrb);
		
	    return(super.update(this));	    
    }; // End of setTime()

	/////////////////////////////////////////////////////////////////////////
	// Animation callback function
	/////////////////////////////////////////////////////////////////////////
	public void animate()
	{
	   if (super.hasAnimation())
	   {		   
	      PsAnimation pa = super.getAnimation();
	      if (pa.isRunning() == true)
	      {
	         pa.stop();  
	      }
	      else
	      {
	         pa.setTimeInterval(0.0, 360.0, 360/number_of_frames, 720/number_of_frames);
	         pa.setTime(angle.getValue());
	         pa.getAnimationPanel().setVisible(false);
	         pa.setRepeat(PsAnimation.BACK_FORTH);
	         pa.start();
	      };
	   };
	}; // End of animate()
	 
	 
	//////////////////////////////////////////////////////////////////////////
	// Initialize variables
	//////////////////////////////////////////////////////////////////////////
	public void init() 
	{
		// Transparency sliders
		f_transparency = new PuDouble("f:", this);
		f_transparency.setDefBounds(0.0, 1.0, 0.01, 0.05);
		f_transparency.setDefValue(f_transparency_ini);
		f_transparency.init();
		f_transparency.setEnabled(true);
		f_transparency.setValue(f_transparency_ini);
		f_transparency.setEnabledConfigButton(false);
		
		g_transparency = new PuDouble("g:", this);
		g_transparency.setDefBounds(0.0, 1.0, 0.01, 0.05);
		g_transparency.setDefValue(g_transparency_ini);
		g_transparency.init();
		g_transparency.setEnabled(true);
		g_transparency.setValue(g_transparency_ini);
		g_transparency.setEnabledConfigButton(false);
		
		a_transparency = new PuDouble("a:", this);
		a_transparency.setDefBounds(0.0, 1.0, 0.01, 0.05);
		a_transparency.setDefValue(a_transparency_ini);
		a_transparency.init();
		a_transparency.setEnabled(true);
		a_transparency.setValue(a_transparency_ini);
		a_transparency.setEnabledConfigButton(false);
		
		b_transparency = new PuDouble("b:", this);
		b_transparency.setDefBounds(0.0, 1.0, 0.01, 0.05);
		b_transparency.setDefValue(b_transparency_ini);
		b_transparency.init();
		b_transparency.setEnabled(true);
		b_transparency.setValue(b_transparency_ini);
		b_transparency.setEnabledConfigButton(false);
		
		// Angle
		angle = new PuDouble("Abertura:", this);
		angle.setDefBounds(0.0, 360.0, 1, 5);
		angle.setDefValue(angle_ini);
		angle.init();
		angle.setEnabled(true);
		angle.setValue(0.0);
		angle.setEnabledConfigButton(false);
		
		// nuLinesPI
		nuLinesPI = new PuInteger("Discretizao 1:", this);
		nuLinesPI.setDefBounds(1, 100, 1, 4);
		nuLinesPI.setDefValue(nuLines);
		nuLinesPI.init();
		nuLinesPI.setEnabled(true);
		nuLinesPI.setValue(nuLines);
		nuLinesPI.setEnabledConfigButton(false);
		
		// nvLinesPI
		nvLinesPI = new PuInteger("Discretizao 2:", this);
		nvLinesPI.setDefBounds(3, 100, 1, 4);
		nvLinesPI.setDefValue(nvLines);
		nvLinesPI.init();
		nvLinesPI.setEnabled(true);
		nvLinesPI.setValue(nvLines);
		nvLinesPI.setEnabledConfigButton(false);
		
		// Meridians
	    meridianf = new PgPolygon(3);
	    meridiang = new PgPolygon(3);
	    meridiana = new PgPolygon(3);
	    meridiana.setNumVertices(2);	  
	    meridianb = new PgPolygon(3);
	    meridianb.setNumVertices(2);
	    
	    // Surfaces of Revolution f
	    // Color fc = new Color(200, 255, 0);
	    ssrf = new SurfaceOfRevolution(3);
	    ssrf.showTransparency(true);
	    ssrf.setTransparency(f_transparency_ini);
	    ssrf.setMeridian(meridianf);
	    ssrf.setDiscr(nvLines);	    
	    ssrf.setGlobalElementColor(new Color(f_R_ini, f_G_ini, f_B_ini));
	    ssrf.showEdges(f_show_edges);
	    ssrg = new SurfaceOfRevolution(3);
	    ssrg.showTransparency(true);
	    ssrg.setTransparency(g_transparency_ini);
	    ssrg.setMeridian(meridiang);
	    ssrg.setDiscr(nvLines);	    
	    ssrg.setGlobalElementColor(new Color(g_R_ini, g_G_ini, g_B_ini));
	    ssrg.showEdges(g_show_edges);	    
	    ssra = new SurfaceOfRevolution(3);
	    ssra.showTransparency(true);
	    ssra.setTransparency(a_transparency_ini);
	    ssra.setMeridian(meridiana);
	    ssra.setDiscr(nvLines);
	    ssra.setGlobalElementColor(new Color(a_R_ini, a_G_ini, a_B_ini));
	    ssra.showEdges(a_show_edges);	    
	    ssrb = new SurfaceOfRevolution(3);
	    ssrb.showTransparency(true);
	    ssrb.setTransparency(b_transparency_ini);
	    ssrb.setMeridian(meridianb);
	    ssrb.setDiscr(nvLines);
	    ssrb.setGlobalElementColor(new Color(b_R_ini, b_G_ini, b_B_ini));
	    ssrb.showEdges(b_show_edges);	    
	    
	    // Parser 
	    myParserF = new JEP();
	    myParserF.initFunTab(); // clear the contents of the function table
	    myParserF.setTraverse(true);
	    myParserF.addStandardFunctions();
	    myParserF.addStandardConstants();
	    myParserF.addFunction("max", new Max()); // Add the custom function
	    myParserF.addFunction("min", new Min()); // Add the custom function
	    myParserF.addFunction("se", new Conditional()); // Add the custom function
	    myParserF.addFunction("Se", new Conditional()); // Add the custom function

	    myParserF.setImplicitMul(true);
	    myParserF.initSymTab(); // clear the contents of the symbol table
	    myParserF.addComplex(); // among other things adds i to the symbol table
	    myParserF.addVariable("x", 0);	    

	    myParserN = new JEP();
	    myParserN.initFunTab(); // clear the contents of the function table
	    myParserN.setTraverse(true);
	    myParserN.addStandardFunctions();
	    myParserN.addStandardConstants();
	    myParserN.addFunction("max", new Max()); // Add the custom function
	    myParserN.addFunction("min", new Min()); // Add the custom function
	    myParserN.addFunction("se", new Conditional()); // Add the custom function
	    myParserN.addFunction("Se", new Conditional()); // Add the custom function

	    myParserN.setImplicitMul(true);
	    myParserN.initSymTab(); // clear the contents of the symbol table
	    myParserN.addComplex(); // among other things adds i to the symbol table
	    
	    // Colors
	    f_pc = new PdColor("f:", this);
	    f_pc.setColor(new Color(f_R_ini, f_G_ini, f_B_ini));
	    f_pc_IP = new PsPanel();	   
	    f_pc_IP = f_pc.newInspector(PsPanel.INFO_EXT);
	    
	    g_pc = new PdColor("g:", this);
	    g_pc.setColor(new Color(g_R_ini, g_G_ini, g_B_ini));
	    g_pc_IP = new PsPanel();	   
	    g_pc_IP = g_pc.newInspector(PsPanel.INFO_EXT);     	    

	    a_pc = new PdColor("a:", this);
	    a_pc.setColor(new Color(a_R_ini, a_G_ini, a_B_ini));
	    a_pc_IP = new PsPanel();	   
	    a_pc_IP = a_pc.newInspector(PsPanel.INFO_EXT);
	    
	    b_pc = new PdColor("b:", this);
	    b_pc.setColor(new Color(b_R_ini, b_G_ini, b_B_ini));
	    b_pc_IP = new PsPanel();	   
	    b_pc_IP = b_pc.newInspector(PsPanel.INFO_EXT);     	    	    	    
	    
	    updateSurface();
	    
        // Axes
	    axis1 = new PgPolygonSet(3); // This is a 3D object
        axis1.setNumVertices(2);
        axis1.setNumPolygons(1);       
        axis1.getVertex(0).setName("");
        axis1.getVertex(1).setName("y");
        axis1.showVertexLabels(true);
        axis1.setPolygon(0, 0, 1);
        axis1.setGlobalPolygonColor(Color.BLUE);
        axis1.setGlobalVertexColor(Color.BLUE);
        axis1.setPolygonSize(0, 0.5);
        axis1.showPolygonSizes(true);
        axis1.showPolygonEndArrow(true);
        
	    axis2 = new PgPolygonSet(3); // This is a 3D object
        axis2.setNumVertices(2);
        axis2.setNumPolygons(1);       
        axis2.getVertex(0).setName("");
        axis2.getVertex(1).setName("z");
        axis2.showVertexLabels(true);
        axis2.setPolygon(0, 0, 1);
        axis2.setGlobalPolygonColor(Color.BLUE);
        axis2.setGlobalVertexColor(Color.BLUE);
        axis2.setPolygonSize(0, 0.5);
        axis2.showPolygonSizes(true);
        axis2.showPolygonEndArrow(true);       	    

	    axis3 = new PgPolygonSet(3); // This is a 3D object
        axis3.setNumVertices(2);
        axis3.setNumPolygons(1);       
        axis3.getVertex(0).setName("");
        axis3.getVertex(1).setName("x");
        axis3.showVertexLabels(true);
        axis3.setPolygon(0, 0, 1);
        axis3.setGlobalPolygonColor(Color.BLUE);
        axis3.setGlobalVertexColor(Color.BLUE);
        axis3.setPolygonSize(0, 0.5);
        axis3.showPolygonSizes(true);
        axis3.showPolygonEndArrow(true);    
        
        updateAxes();
	}; // End of init()

	//////////////////////////////////////////////////////////////////////////
	// Update all
	//////////////////////////////////////////////////////////////////////////
	public int updateSurface() 
	{
		String expression;
        double x;
        double v;		
		
		// Check a 
        expression = a;
        expression = a.toLowerCase();
        expression = expression.replaceAll("sen", "sin"); 
        expression = expression.replaceAll("tg", "tan"); 
        expression = expression.replaceAll("0pi", "0*pi"); 
        expression = expression.replaceAll("1pi", "1*pi"); 
        expression = expression.replaceAll("2pi", "2*pi"); 
        expression = expression.replaceAll("3pi", "3*pi"); 
        expression = expression.replaceAll("4pi", "4*pi"); 
        expression = expression.replaceAll("5pi", "5*pi"); 
        expression = expression.replaceAll("6pi", "6*pi"); 
        expression = expression.replaceAll("7pi", "7*pi"); 
        expression = expression.replaceAll("8pi", "8*pi"); 
        expression = expression.replaceAll("9pi", "9*pi"); 
        expression = expression.replaceAll("pi", "3.141592653589793");
      
        myParserN.parseExpression(expression);
      
        if (myParserN.hasError() == true)
        {
           System.out.println("Error in expression for a (" + expression + "): " + myParserF.getErrorInfo());
           return(1);
        };
        
        xMin = myParserN.getValue();
        
		// Check b 
        expression = b;
        expression = b.toLowerCase();
        expression = expression.replaceAll("sen", "sin"); 
        expression = expression.replaceAll("tg", "tan"); 
        expression = expression.replaceAll("0pi", "0*pi"); 
        expression = expression.replaceAll("1pi", "1*pi"); 
        expression = expression.replaceAll("2pi", "2*pi"); 
        expression = expression.replaceAll("3pi", "3*pi"); 
        expression = expression.replaceAll("4pi", "4*pi"); 
        expression = expression.replaceAll("5pi", "5*pi"); 
        expression = expression.replaceAll("6pi", "6*pi"); 
        expression = expression.replaceAll("7pi", "7*pi"); 
        expression = expression.replaceAll("8pi", "8*pi"); 
        expression = expression.replaceAll("9pi", "9*pi"); 
        expression = expression.replaceAll("pi", "3.14159");
      
        myParserN.parseExpression(expression);
      
        if (myParserN.hasError() == true)
        {
           System.out.println("Error in expression for b (" + expression + "): " + myParserF.getErrorInfo());
           return(2);
        };
        
        xMax = myParserN.getValue();
        
        xStp = (xMax - xMin)/nuLines;
		
		// Check f 
        expression = f.toLowerCase();
        expression = expression.replaceAll("sen", "sin"); 
        expression = expression.replaceAll("tg", "tan"); 
        expression = expression.replaceAll("0pi", "0*pi"); 
        expression = expression.replaceAll("1pi", "1*pi"); 
        expression = expression.replaceAll("2pi", "2*pi"); 
        expression = expression.replaceAll("3pi", "3*pi"); 
        expression = expression.replaceAll("4pi", "4*pi"); 
        expression = expression.replaceAll("5pi", "5*pi"); 
        expression = expression.replaceAll("6pi", "6*pi"); 
        expression = expression.replaceAll("7pi", "7*pi"); 
        expression = expression.replaceAll("8pi", "8*pi"); 
        expression = expression.replaceAll("9pi", "9*pi"); 
        expression = expression.replaceAll("pi", "3.14159");
      
        myParserF.parseExpression(expression);
      
        if (myParserF.hasError() == true)
        {
           System.out.println("Error in expression f (" + expression + "): " + myParserF.getErrorInfo());
           return(3);
        };
        
        meridianf.setNumVertices(nuLines + 1);
        
        for (int i = 0; i <= nuLines; i++)
        {
        	x = xMin + i*xStp;
        	myParserF.addVariable("x", x);
        	v = myParserF.getValue();
            if ((myParserF.hasError() == true) || (Double.isInfinite(v) == true) ||
            	(Double.isNaN(v) == true))
            {
               System.out.println("Error while evaluating f (" + expression + "): " + myParserF.getErrorInfo());
               return(4);
            };
            meridianf.setVertex(i, 0, v, x);            
        }
        
		// Check g 
        expression = g.toLowerCase();
        expression = expression.replaceAll("sen", "sin"); 
        expression = expression.replaceAll("tg", "tan"); 
        expression = expression.replaceAll("0pi", "0*pi"); 
        expression = expression.replaceAll("1pi", "1*pi"); 
        expression = expression.replaceAll("2pi", "2*pi"); 
        expression = expression.replaceAll("3pi", "3*pi"); 
        expression = expression.replaceAll("4pi", "4*pi"); 
        expression = expression.replaceAll("5pi", "5*pi"); 
        expression = expression.replaceAll("6pi", "6*pi"); 
        expression = expression.replaceAll("7pi", "7*pi"); 
        expression = expression.replaceAll("8pi", "8*pi"); 
        expression = expression.replaceAll("9pi", "9*pi"); 
        expression = expression.replaceAll("pi", "3.14159");
      
        myParserF.parseExpression(expression);
      
        if (myParserF.hasError() == true)
        {
           System.out.println("Error in expression g (" + expression + "): " + myParserF.getErrorInfo());
           return(5);
        };
        
        meridiang.setNumVertices(nuLines + 1);
        
        for (int i = 0; i <= nuLines; i++)
        {
        	x = xMin + i*xStp;
        	myParserF.addVariable("x", x);
        	v = myParserF.getValue();
            if ((myParserF.hasError() == true) || (Double.isInfinite(v) == true) ||
                (Double.isNaN(v) == true))
            {
               System.out.println("Error while evaluating g (" + expression + "): " + myParserF.getErrorInfo());
               return(6);
            };
            meridiang.setVertex(i, 0, v, x);            
        }
        
        // Setup a
        meridiana.setVertex(0, 0, meridiang.getVertex(0).getEntry(1), xMin);
        meridiana.setVertex(1, 0, meridianf.getVertex(0).getEntry(1), xMin);
        
        // Setup b
        meridianb.setVertex(0, 0, meridiang.getVertex(nuLines).getEntry(1), xMax);
        meridianb.setVertex(1, 0, meridianf.getVertex(nuLines).getEntry(1), xMax);
        
        ssrf.showEdges(f_show_edges);
        ssrg.showEdges(g_show_edges);
        ssra.showEdges(a_show_edges);
        ssrb.showEdges(b_show_edges);
        
        ssrf.compute();
        ssrg.compute();
        ssra.compute();
        ssrb.compute();
        
        ssrf.update(ssrf);
        ssrg.update(ssrg);
        ssra.update(ssra);
        ssrb.update(ssrb);        
       		
		return(0);
	} // End of updateSurface()
	
	//////////////////////////////////////////////////////////////////////////
	// Update f
	//////////////////////////////////////////////////////////////////////////
	public int updateF() 
	{
		String expression;
        double x;
        double v;		
        
		// Check f 
        expression = f.toLowerCase();
        expression = expression.replaceAll("sen", "sin"); 
        expression = expression.replaceAll("tg", "tan"); 
        expression = expression.replaceAll("0pi", "0*pi"); 
        expression = expression.replaceAll("1pi", "1*pi"); 
        expression = expression.replaceAll("2pi", "2*pi"); 
        expression = expression.replaceAll("3pi", "3*pi"); 
        expression = expression.replaceAll("4pi", "4*pi"); 
        expression = expression.replaceAll("5pi", "5*pi"); 
        expression = expression.replaceAll("6pi", "6*pi"); 
        expression = expression.replaceAll("7pi", "7*pi"); 
        expression = expression.replaceAll("8pi", "8*pi"); 
        expression = expression.replaceAll("9pi", "9*pi"); 
        expression = expression.replaceAll("pi", "3.14159");
      
        myParserF.parseExpression(expression);
      
        if (myParserF.hasError() == true)
        {
           System.out.println("Error in expression f (" + expression + "): " + myParserF.getErrorInfo());
           return(3);
        };
        
        meridianf.setNumVertices(nuLines + 1);
        
        for (int i = 0; i <= nuLines; i++)
        {
        	x = xMin + i*xStp;
        	myParserF.addVariable("x", x);
        	v = myParserF.getValue();
            if ((myParserF.hasError() == true) || (Double.isInfinite(v) == true) ||
                (Double.isNaN(v) == true))
            {
               System.out.println("Error while evaluating f (" + expression + "): " + myParserF.getErrorInfo());
               return(4);
            };
            meridianf.setVertex(i, 0, v, x);            
        }
                
        // Setup a
        meridiana.setVertex(0, 0, meridiang.getVertex(0).getEntry(1), xMin);
        meridiana.setVertex(1, 0, meridianf.getVertex(0).getEntry(1), xMin);
        
        // Setup b
        meridianb.setVertex(0, 0, meridiang.getVertex(nuLines).getEntry(1), xMax);
        meridianb.setVertex(1, 0, meridianf.getVertex(nuLines).getEntry(1), xMax);
        
        ssrf.showEdges(f_show_edges);
        ssra.showEdges(a_show_edges);
        ssrb.showEdges(b_show_edges);
        
        ssrf.compute();
        ssra.compute();
        ssrb.compute();
        
        ssrf.update(ssrf);
        ssra.update(ssra);
        ssrb.update(ssrb);
		
		return(0);
	} // End of updateF()
		
	//////////////////////////////////////////////////////////////////////////
	// Update g
	//////////////////////////////////////////////////////////////////////////
	public int updateG() 
	{
		String expression;
        double x;
        double v;		
        
		// Check g 
        expression = g.toLowerCase();
        expression = expression.replaceAll("sen", "sin"); 
        expression = expression.replaceAll("tg", "tan"); 
        expression = expression.replaceAll("0pi", "0*pi"); 
        expression = expression.replaceAll("1pi", "1*pi"); 
        expression = expression.replaceAll("2pi", "2*pi"); 
        expression = expression.replaceAll("3pi", "3*pi"); 
        expression = expression.replaceAll("4pi", "4*pi"); 
        expression = expression.replaceAll("5pi", "5*pi"); 
        expression = expression.replaceAll("6pi", "6*pi"); 
        expression = expression.replaceAll("7pi", "7*pi"); 
        expression = expression.replaceAll("8pi", "8*pi"); 
        expression = expression.replaceAll("9pi", "9*pi"); 
        expression = expression.replaceAll("pi", "3.14159");
      
        myParserF.parseExpression(expression);
      
        if (myParserF.hasError() == true)
        {
           System.out.println("Error in expression g (" + expression + "): " + myParserF.getErrorInfo());
           return(5);
        };
        
        meridiang.setNumVertices(nuLines + 1);
        
        for (int i = 0; i <= nuLines; i++)
        {
        	x = xMin + i*xStp;
        	myParserF.addVariable("x", x);
        	v = myParserF.getValue();
            if ((myParserF.hasError() == true) || (Double.isInfinite(v) == true) ||
                (Double.isNaN(v) == true))
            {
               System.out.println("Error while evaluating g (" + expression + "): " + myParserF.getErrorInfo());
               return(6);
            };
            meridiang.setVertex(i, 0, v, x);            
        }
                
        // Setup a
        meridiana.setVertex(0, 0, meridiang.getVertex(0).getEntry(1), xMin);
        meridiana.setVertex(1, 0, meridianf.getVertex(0).getEntry(1), xMin);
        
        // Setup b
        meridianb.setVertex(0, 0, meridiang.getVertex(nuLines).getEntry(1), xMax);
        meridianb.setVertex(1, 0, meridianf.getVertex(nuLines).getEntry(1), xMax);

        ssrg.showEdges(g_show_edges);
        ssra.showEdges(a_show_edges);
        ssrb.showEdges(b_show_edges);
        
        ssrg.compute();
        ssra.compute();
        ssrb.compute();
        
        ssrg.update(ssrg);
        ssra.update(ssra);
        ssrb.update(ssrb);
		
		return(0);
	} // End of updateG()

	//////////////////////////////////////////////////////////////////////////
	// Update axes
	//////////////////////////////////////////////////////////////////////////	
	public void updateAxes()
	{
        double xMin;
        double yMin;
        double zMin;
        double xMax;
        double yMax;
        double zMax;
        
		zMin = (ssrf.getBounds())[0].getEntry(2);
		yMax = (ssrf.getBounds())[1].getEntry(1);
		zMax = (ssrf.getBounds())[1].getEntry(2);

		zMin = Math.min(zMin,(ssrg.getBounds())[0].getEntry(2));
		yMax = Math.max(yMax,(ssrg.getBounds())[1].getEntry(1));
		zMax = Math.max(zMax,(ssrg.getBounds())[1].getEntry(2));

		zMin = Math.min(zMin,(ssra.getBounds())[0].getEntry(2));
		yMax = Math.max(yMax,(ssra.getBounds())[1].getEntry(1));
		zMax = Math.max(zMax,(ssra.getBounds())[1].getEntry(2));

		zMin = Math.min(zMin,(ssrb.getBounds())[0].getEntry(2));
		yMax = Math.max(yMax,(ssrb.getBounds())[1].getEntry(1));
		zMax = Math.max(zMax,(ssrb.getBounds())[1].getEntry(2));
		
		if (zMin > 0)
		{
			zMin = 0;
		}
		
		if (zMax < 0)
		{
			zMax = 0;
		}
		
		yMax = Math.abs(yMax);
		yMin = -yMax;
		xMax = yMax;
		xMin = yMin;
		
		double deltaX = 0.2*xMax;
		double deltaY = 0.2*yMax;
		double deltaZ = 0.1*Math.abs(zMax - zMin);
		double delta = Math.max(deltaX, Math.max(deltaY, deltaZ));
		
		xMin = xMin - delta;
		xMax = xMax + delta;
		yMin = yMin - delta;
		yMax = yMax + delta;
		zMin = zMin - delta;
		zMax = zMax + delta;
		
		// Axes
	    axis1.setVertex(0, new PdVector(xMin, 0, 0));
	    axis1.setVertex(1, new PdVector(xMax, 0, 0));
        axis1.getVertex(0).setName("");
        axis1.getVertex(1).setName("y");
        axis1.showVertexLabels(true);
        axis1.setPolygon(0, 0, 1);
        axis1.setGlobalPolygonColor(Color.BLUE);
        axis1.setGlobalVertexColor(Color.BLUE);
        axis1.setGlobalVertexSize(0.0);
        axis1.setPolygonSize(0, 0.5);
        axis1.showPolygonSizes(true);
        axis1.showPolygonEndArrow(true);
        axis1.update(axis1);
        
	    axis2.setVertex(0, new PdVector(0, yMin, 0));
	    axis2.setVertex(1, new PdVector(0, yMax, 0));
        axis2.getVertex(0).setName("");
        axis2.getVertex(1).setName("z");
        axis2.showVertexLabels(true);
        axis2.setPolygon(0, 0, 1);
        axis2.setGlobalPolygonColor(Color.BLUE);
        axis2.setGlobalVertexColor(Color.BLUE);
        axis2.setGlobalVertexSize(0.0);
        axis2.setPolygonSize(0, 0.5);
        axis2.showPolygonSizes(true);
        axis2.showPolygonEndArrow(true);
        axis2.update(axis2);		

	    axis3.setVertex(0, new PdVector(0, 0, zMin));
	    axis3.setVertex(1, new PdVector(0, 0, zMax));
        axis3.getVertex(0).setName("");
        axis3.getVertex(1).setName("x");
        axis3.showVertexLabels(true);
        axis3.setPolygon(0, 0, 1);
        axis3.setGlobalPolygonColor(Color.BLUE);
        axis3.setGlobalVertexColor(Color.BLUE);
        axis3.setGlobalVertexSize(0.0);
        axis3.setPolygonSize(0, 0.5);
        axis3.showPolygonSizes(true);
        axis3.showPolygonEndArrow(true);
        axis3.update(axis3);		        
	}

	//////////////////////////////////////////////////////////////////////////
	// Things to do when the class is started
	//////////////////////////////////////////////////////////////////////////
	public void start() 
	{
		super.start();
		
		addGeometry(ssrf);
		addGeometry(ssrg);
		addGeometry(ssra);
		addGeometry(ssrb);
		if (x_cb_ini == true)
		{
		   addGeometry(axis3);
		}
		if (y_cb_ini == true)
		{
		   addGeometry(axis1);
		}
		if (z_cb_ini == true)
		{
		   addGeometry(axis2);
		}
		update(this);

		disp = getDisplay(); // The PwUnfold needs a reference to the
		                     // display in order to work properly.

		PdVector pos = new PdVector(1, 0, 0);
		PdVector vdi = new PdVector(1, 0, 0);
		cam = disp.getCamera();
		cam.setPosition(pos);
		cam.setViewDir(vdi);
		cam.setRoll(90);
		cam.update(cam);
		disp.fit();		
		
		if (i_cb_ini == false)
		{
  	       // disp.setLightingModel(PvLightIf.LIGHT_AMBIENT);
		   disp.showDepthcue(false);
	       disp.update(disp);			
		}
		else
		{
	  	   // disp.setLightingModel(PvLightIf.LIGHT_DIRECTION);
			disp.showDepthcue(true);
		   disp.update(disp);						
		}
		
        // Key events
        internalKeyListener = new KeyAdapter() 
        {
            public void keyPressed(KeyEvent keyEvent) 
            {
           	   if (keyEvent.getKeyCode() == KeyEvent.VK_1)
        	   {
           	      if (isShowing_f == true)
           		  {
           			isShowing_f = false;
           			disp.removeGeometry(ssrf);
           			disp.update(disp);
           		  }
           		  else
           		  {
             		isShowing_f = true;
               		disp.addGeometry(ssrf);  
               		disp.update(disp);
           		  }
        	   };
        	   
           	   if (keyEvent.getKeyCode() == KeyEvent.VK_2)
        	   {
           	      if (isShowing_g == true)
           		  {
           			isShowing_g = false;
           			disp.removeGeometry(ssrg);
           			disp.update(disp);           			
           		  }
           		  else
           		  {
             		isShowing_g = true;
               		disp.addGeometry(ssrg);
           			disp.update(disp);               		
           		  }
        	   };        	   
        	   
           	   if (keyEvent.getKeyCode() == KeyEvent.VK_3)
        	   {
           	      if (isShowing_a == true)
           		  {
           			isShowing_a = false;
           			disp.removeGeometry(ssra);
           			disp.update(disp);           			
           		  }
           		  else
           		  {
             		isShowing_a = true;
               		disp.addGeometry(ssra);
           			disp.update(disp);               		
           		  }
        	   };        	   
        	   
           	   if (keyEvent.getKeyCode() == KeyEvent.VK_4)
        	   {
           	      if (isShowing_b == true)
           		  {
           			isShowing_b = false;
           			disp.removeGeometry(ssrb);
           			disp.update(disp);           			
           		  }
           		  else
           		  {
             		isShowing_b = true;
               		disp.addGeometry(ssrb);
           			disp.update(disp);
           		  }
        	   };        	   
            	
         	   if (keyEvent.getKeyCode() == KeyEvent.VK_8)
        	   {
        	 	  fieldOfView -= 0.05;
         		  if (fieldOfView <= 0.0)
        		  {
        			  fieldOfView = 0.0;
        		  };
        		  cam.setFieldOfView(fieldOfView);
        		  cam.update(cam);
        	   };        	  
        	  
        	   if (keyEvent.getKeyCode() == KeyEvent.VK_9)
        	   {
        	      fieldOfView += 0.05;
        		  if (fieldOfView >= 2.0)
        		  {
        			  fieldOfView = 2.0;
        		  };
        		  cam.setFieldOfView(fieldOfView);
        		  cam.update(cam);
        	   };        	            	   
            };
        };
        disp.addKeyListener(internalKeyListener);		
	}; // End of start()

	/////////////////////////////////////////////////////////////////////////
	// Update the class whenever a child has changed. Method is usually
	// invoked from the children.
	/////////////////////////////////////////////////////////////////////////
	public boolean update(Object event) 
	{
		if (event == null)
		{
			return (true);
		};

		// Solid transparency
		if (event == f_transparency) 
		{			
			ssrf.setTransparency(f_transparency.getValue());
			ssrf.update(ssrf);
			
			return (true);
		};
		
		if (event == g_transparency) 
		{			
			ssrg.setTransparency(g_transparency.getValue());
			ssrg.update(ssrg);
			
			return (true);
		};

		if (event == a_transparency) 
		{			
			ssra.setTransparency(a_transparency.getValue());
			ssra.update(ssra);
			
			return (true);
		};

		if (event == b_transparency) 
		{			
			ssrb.setTransparency(b_transparency.getValue());
			ssrb.update(ssrb);
			
			return (true);
		};
		
		
		if (event == angle) 
		{			
			double a = (360 - angle.getValue())*Math.PI/180.0;
			ssrf.setAngle(a);
			ssrg.setAngle(a);
			ssra.setAngle(a);
			ssrb.setAngle(a);
			ssrf.update(ssrf);
			ssrg.update(ssrg);
			ssra.update(ssra);
			ssrb.update(ssrb);
			
			return (true);
		};
		
		if ((event == nuLinesPI) || (event == nvLinesPI)) 
		{			
			nuLines = nuLinesPI.getValue();
			nvLines = nvLinesPI.getValue();
			ssrf.setDiscr(nvLines);
			ssrg.setDiscr(nvLines);
			ssra.setDiscr(nvLines);
			ssrb.setDiscr(nvLines);
			updateSurface();
			ssrf.update(ssrf);
			ssrg.update(ssrg);
			ssra.update(ssra);
			ssrb.update(ssrb);
			
			return (true);
		};
				
		if (event == f_pc)
		{		
			ssrf.setGlobalElementColor(f_pc.getColor());
			ssrf.update(ssrf);
			
			return (true);			
		}
		
		if (event == g_pc)
		{		
			ssrg.setGlobalElementColor(g_pc.getColor());
			ssrg.update(ssrg);
			
			return (true);			
		}
		
		if (event == a_pc)
		{		
			ssra.setGlobalElementColor(a_pc.getColor());
			ssra.update(ssra);
			
			return (true);			
		}
		
		if (event == b_pc)
		{		
			ssrb.setGlobalElementColor(b_pc.getColor());
			ssrb.update(ssrb);
			
			return (true);			
		}
		
		return (super.update(event));
	}; // End of update()
}; // End of PjRevolutionX classs
