///////////////////////////////////////////////////////////////////////////////
// PjRevolutionX Interface class
//
// Last update: 13/01/2008
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Native JAVA imports
///////////////////////////////////////////////////////////////////////////////
import java.awt.Button;
import java.awt.Checkbox;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.Panel;
import java.awt.TextField;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

///////////////////////////////////////////////////////////////////////////////
// JavaView imports
///////////////////////////////////////////////////////////////////////////////
import jv.object.PsPanel;
import jv.objectGui.PsTabPanel;
import jv.object.PsUpdateIf;
import jv.project.PjProject_IP;
import jv.project.PvLightIf;

///////////////////////////////////////////////////////////////////////////////
// Info panel for graph tomography applet
///////////////////////////////////////////////////////////////////////////////
public class PjRevolutionX_IP extends PjProject_IP implements ActionListener, ItemListener
{
   protected PjRevolutionX m_pjRevolutionX;

   protected Panel m_ftPanel; // Transparency panel
   protected Panel m_gtPanel; // Transparency panel
   protected Panel m_atPanel; // Transparency panel
   protected Panel m_btPanel; // Transparency panel
   protected Panel m_aPanel; // Angle panel
   protected Panel m_uPanel; // nuLinesPI panel
   protected Panel m_vPanel; // nvLinesPI panel  
   
   protected TextField m_fTextField;                    // Text field for f
   protected TextField m_gTextField;                    // Text field for g
   protected TextField m_aTextField;                 // Text field for uMin
   protected TextField m_bTextField;                 // Text field for uMax
   
   protected Label msgLabel;   
  
   protected Button updateButton;
   protected Button animateButton;
   
   protected Color errorColor;
   
   protected Checkbox x_cb;
   protected Checkbox y_cb;
   protected Checkbox z_cb;
   protected Checkbox f_cb;
   protected Checkbox g_cb;
   protected Checkbox a_cb;
   protected Checkbox b_cb;
   protected Checkbox i_cb;
   
   protected PsTabPanel tabPanel;

   protected int idiom = 0;
   protected String[] title_trans = {"Superfcies e Slidos de Revoluo (X)",
                                     "Surfaces and Solids of Revolution (X)"};

   protected String[] par_update_trans = {"Atualizar!", "Update!"};
   protected String[] par_animate_trans = {"Animar!", "Animate!"};
   protected String[] par_stop_trans = {"Parar!", "Stop!"};
   protected String[] par_angle_trans = {"Abertura:", "Opening:"};
   protected String[] par_nuLinesPI_trans = {"Discretizao 1:", "Discretization 1:"};  
   protected String[] par_nvLinesPI_trans = {"Discretizao 2:", "Discretization 2:"};   
   
   protected String[] cfg_transparency_trans = {"Transparncias",
                                                "Transparencies"};      
   protected String[] cfg_color_trans = {"Cores", "Colors"};
   
   protected String[] cfg_f_transparency_trans = {"f:", "f:"};
   protected String[] cfg_g_transparency_trans = {"g:", "g:"};
   protected String[] cfg_a_transparency_trans = {"a:", "a:"};
   protected String[] cfg_b_transparency_trans = {"b:", "b:"};   
   
   protected String[] cfg_f_pc_trans = {"f: ", "f:"};
   protected String[] cfg_g_pc_trans = {"g: ", "g:"};
   protected String[] cfg_a_pc_trans = {"a: ", "a:"};
   protected String[] cfg_b_pc_trans = {"b: ", "b:"};
   
   protected String[] ele_axes_trans = {"Eixos", "Axes"};
   protected String[] ele_edges_trans = {"Malhas", "Malhas"};
   protected String[] ele_lighting_trans = {"Iluminao", "Lighting"};                   
   protected String[] ele_x_cb_trans = {"Exibir eixo x     ", "Show x axis     "};
   protected String[] ele_y_cb_trans = {"Exibir eixo y     ", "Show y axis     "};
   protected String[] ele_z_cb_trans = {"Exibir eixo z", "Show z axis"};
   protected String[] ele_f_cb_trans = {"Exibir malha de f", "Show grid of f"};
   protected String[] ele_g_cb_trans = {"Exibir malha de g", "Show grid of g"};
   protected String[] ele_a_cb_trans = {"Exibir malha de a", "Show grid of a"};
   protected String[] ele_b_cb_trans = {"Exibir malha de b", "Show grid of b"};
   protected String[] ele_i_cb_trans = {"Habilitar iluminao direcionada (no uniforme)", "Enable lighting direction (non-uniform)"};   

   protected String[] par_panel_trans = {"Parmetros", "Parameters"};   
   protected String[] cfg_panel_trans = {"Configurao", "Configuration"};
   protected String[] ele_panel_trans = {"Elementos", "Elements"};
   
   protected String[] rc_invalid_input_trans = {"Entrada invlida!", "Invalid input!"};
   protected String[] rc_f_eval_trans = {"f no est definida para algum x entre a e b! ", "f is not defined for some x between a and b!"};
   protected String[] rc_g_eval_trans = {"g no est definida para algum x entre a e b! ", "g is not defined for some x between a and b!"};

   
   ///////////////////////////////////////////////////////////////////////////
   // Default constructor
   ///////////////////////////////////////////////////////////////////////////
   public PjRevolutionX_IP()
   {
      super();

      if (getClass() == PjRevolutionX_IP.class)
      {
         init();
      };
   }; // End of PjRevolutionX_IP()


   //////////////////////////////////////////////////////////////////////////
   // Setup the main control panel
   //////////////////////////////////////////////////////////////////////////
   public void init()
   {
      super.init();

      // Title
      addTitle(title_trans[idiom]);

      // add(new Label(" "));
      {
          Label l = new Label(" ");
          l.setFont(new Font("dialog", Font.PLAIN, 2));
          add(l);
      }
                 
      errorColor = new Color(255, 119, 119);

      tabPanel = new PsTabPanel();
      add(tabPanel);
   }; // End of init()


   //////////////////////////////////////////////////////////////////////////
   // Setup parent and add the level slider
   //////////////////////////////////////////////////////////////////////////
   public void setParent(PsUpdateIf parent)
   {
      super.setParent(parent);
      m_pjRevolutionX = (PjRevolutionX) parent;
      idiom = m_pjRevolutionX.idiom;

      // Parameters panel
      setTitle(title_trans[idiom]);
      
      PsPanel parPanel = new PsPanel();
      
      parPanel.add(new Label(" "));           
            
      PsPanel m_mePsPanel = new PsPanel();
      GridBagLayout bl = new GridBagLayout();
      m_mePsPanel.setLayout(bl);      

      Label lf = new Label("f(x) =");
      lf.setFont(new Font("monospaced", Font.PLAIN, 15));
      {
	      GridBagConstraints c = new GridBagConstraints();
	      c.fill = GridBagConstraints.NONE;
	      c.anchor = GridBagConstraints.EAST;
	      c.gridx = 0;
	      c.gridy = 0;
	      c.gridwidth = 1;
	      c.gridheight = 1;
	      c.weightx = 1.0;
	      bl.setConstraints(lf, c);
      }
      m_mePsPanel.add(lf);

      m_fTextField = new TextField();
      m_fTextField.addActionListener(this);
      {
	      GridBagConstraints c = new GridBagConstraints();
	      c.fill = GridBagConstraints.HORIZONTAL;
	      c.gridx = 1;
	      c.gridy = 0;
	      c.gridwidth = 1;
	      c.gridheight = 1;
	      c.weightx = 40.0;
	      bl.setConstraints(m_fTextField, c);
      }
      m_mePsPanel.add(m_fTextField);
                      
      Label lg = new Label("g(x) =");
      lg.setFont(new Font("monospaced", Font.PLAIN, 15));      
      {
	      GridBagConstraints c = new GridBagConstraints();
	      c.fill = GridBagConstraints.NONE;
	      c.anchor = GridBagConstraints.EAST;
	      c.gridx = 0;
	      c.gridy = 1;
	      c.gridwidth = 1;
	      c.gridheight = 1;
	      c.weightx = 1.0;
	      bl.setConstraints(lg, c);
      }
      m_mePsPanel.add(lg);

      m_gTextField = new TextField();
      m_gTextField.addActionListener(this);
      {
	      GridBagConstraints c = new GridBagConstraints();
	      c.fill = GridBagConstraints.HORIZONTAL;
	      c.gridx = 1;
	      c.gridy = 1;
	      c.gridwidth = 1;
	      c.gridheight = 1;
	      c.weightx = 40.0;
	      bl.setConstraints(m_gTextField, c);
      }
      m_mePsPanel.add(m_gTextField);
      
      Label la = new Label("a    =");      
      la.setFont(new Font("monospaced", Font.PLAIN, 15));
      {
	      GridBagConstraints c = new GridBagConstraints();
	      c.fill = GridBagConstraints.NONE;
	      c.anchor = GridBagConstraints.EAST;
	      c.gridx = 0;
	      c.gridy = 2;
	      c.gridwidth = 1;
	      c.gridheight = 1;
	      c.weightx = 1.0;
	      bl.setConstraints(la, c);
      }
      m_mePsPanel.add(la);
            
      m_aTextField = new TextField();
      m_aTextField.addActionListener(this);
      {
	      GridBagConstraints c = new GridBagConstraints();
	      c.fill = GridBagConstraints.HORIZONTAL;
	      c.gridx = 1;
	      c.gridy = 2;
	      c.gridwidth = 1;
	      c.gridheight = 1;
	      c.weightx = 20.0;
	      bl.setConstraints(m_aTextField, c);
      }
      m_mePsPanel.add(m_aTextField);
      
      Label lb = new Label("b    =");  
      lb.setFont(new Font("monospaced", Font.PLAIN, 15));
      {
	      GridBagConstraints c = new GridBagConstraints();
	      c.fill = GridBagConstraints.NONE;
	      c.anchor = GridBagConstraints.EAST;
	      c.gridx = 0;
	      c.gridy = 3;
	      c.gridwidth = 1;
	      c.gridheight = 1;
	      c.weightx = 1.0;
	      bl.setConstraints(lb, c);
      }
      m_mePsPanel.add(lb);
      
      m_bTextField = new TextField();
      m_bTextField.addActionListener(this);
      {
	      GridBagConstraints c = new GridBagConstraints();
	      c.fill = GridBagConstraints.HORIZONTAL;
	      c.gridx = 1;
	      c.gridy = 3;
	      c.gridwidth = 1;
	      c.gridheight = 1;
	      c.weightx = 20.0;
	      bl.setConstraints(m_bTextField, c);
      }
      m_mePsPanel.add(m_bTextField);      
      
      parPanel.add(m_mePsPanel);
      
      {
          Label l = new Label(" ");
          l.setFont(new Font("dialog", Font.PLAIN, 1));
          parPanel.add(l);
      }      
            
      msgLabel = new Label(" ", Label.CENTER);
      msgLabel.setForeground(Color.RED);
      msgLabel.setFont(new Font("dialog", Font.PLAIN, 12));
      parPanel.add(msgLabel);                       

      {
          Label l = new Label(" ");
          l.setFont(new Font("dialog", Font.PLAIN, 1));
          parPanel.add(l);
      }            
      
      PsPanel m_bPsPanel = new PsPanel();
      GridBagLayout bp = new GridBagLayout();
      m_bPsPanel.setLayout(bp);
      parPanel.add(m_bPsPanel);
      
      {
    	  Label l = new Label(" ");
	      GridBagConstraints c = new GridBagConstraints();
	      c.fill = GridBagConstraints.HORIZONTAL;	      
	      c.gridx = 0;
	      c.gridy = 0;
	      c.weightx = 1.0;
	      bp.setConstraints(l, c);
	      m_bPsPanel.add(l);
      }            
      
      updateButton = new Button(par_update_trans[idiom]);
      updateButton.addActionListener(this);      
      {
	      GridBagConstraints c = new GridBagConstraints();
	      c.fill = GridBagConstraints.HORIZONTAL;	      
	      c.gridx = 1;
	      c.gridy = 0;
	      c.weightx = 20.0;
	      bp.setConstraints(updateButton, c);
      }
      m_bPsPanel.add(updateButton);

      {
    	  Label l = new Label(" ");
	      GridBagConstraints c = new GridBagConstraints();
	      c.fill = GridBagConstraints.HORIZONTAL;	      
	      c.gridx = 2;
	      c.gridy = 0;
	      c.weightx = 1.0;
	      bp.setConstraints(l, c);
	      m_bPsPanel.add(l);
      }            
      
      animateButton = new Button(par_animate_trans[idiom]);
      animateButton.addActionListener(this);    
      {
	      GridBagConstraints c = new GridBagConstraints();
	      c.fill = GridBagConstraints.HORIZONTAL;
	      c.gridx = 3;
	      c.gridy = 0;
	      c.gridwidth = 1;
	      c.gridheight = 1;
	      c.weightx = 20.0;
	      bp.setConstraints(animateButton, c);
      }
      m_bPsPanel.add(animateButton);
      
      {
    	  Label l = new Label(" ");
	      GridBagConstraints c = new GridBagConstraints();
	      c.fill = GridBagConstraints.HORIZONTAL;	      
	      c.gridx = 4;
	      c.gridy = 0;
	      c.weightx = 1.0;
	      bp.setConstraints(l, c);
	      m_bPsPanel.add(l);
      }            
     
      
      {
          Label l = new Label(" ");
          l.setFont(new Font("dialog", Font.PLAIN, 3));
          parPanel.add(l);
      }     
     
      parPanel.addLine(1);      
      
      m_aPanel = new Panel();
      m_aPanel.setLayout(new GridLayout(1, 1));
      parPanel.add(m_aPanel);
      m_aPanel.removeAll();
      m_pjRevolutionX.angle.setName(par_angle_trans[idiom]);
      m_aPanel.add(m_pjRevolutionX.angle.newInspector(PsPanel.INFO_EXT));
      
      parPanel.addLine(1);
      
      m_uPanel = new Panel();
      m_uPanel.setLayout(new GridLayout(1, 1));
      parPanel.add(m_uPanel);
      m_uPanel.removeAll();
      m_pjRevolutionX.nuLinesPI.setName(par_nuLinesPI_trans[idiom]);
      m_uPanel.add(m_pjRevolutionX.nuLinesPI.newInspector(PsPanel.INFO_EXT));

      m_vPanel = new Panel();
      m_vPanel.setLayout(new GridLayout(1, 1));
      parPanel.add(m_vPanel);
      m_vPanel.removeAll();
      m_pjRevolutionX.nvLinesPI.setName(par_nvLinesPI_trans[idiom]);
      m_vPanel.add(m_pjRevolutionX.nvLinesPI.newInspector(PsPanel.INFO_EXT));
            
      tabPanel.addPanel(par_panel_trans[idiom], parPanel);  
      
      parPanel.addLine(1);
      
      // Appareance panel
      PsPanel cfgPanel = new PsPanel();
      
      {
         Label l = new Label(" ");
         l.setFont(new Font("dialog", Font.PLAIN, 6));
         cfgPanel.add(l);
      }
      
      Label transparencyLabel = new Label(cfg_transparency_trans[idiom], Label.CENTER);
      transparencyLabel.setFont(new Font("dialog", Font.BOLD, 13));
      cfgPanel.add(transparencyLabel);   

      {
          Label l = new Label(" ");
          l.setFont(new Font("dialog", Font.PLAIN, 2));
          cfgPanel.add(l);
      }
            
      m_ftPanel = new Panel();
      m_ftPanel.setLayout(new GridLayout(1, 1));
      cfgPanel.add(m_ftPanel);
      m_ftPanel.removeAll();
      m_pjRevolutionX.f_transparency.setName(cfg_f_transparency_trans[idiom]);
      m_ftPanel.add(m_pjRevolutionX.f_transparency.newInspector(PsPanel.INFO_EXT));
      
      m_gtPanel = new Panel();
      m_gtPanel.setLayout(new GridLayout(1, 1));
      cfgPanel.add(m_gtPanel);
      m_gtPanel.removeAll();
      m_pjRevolutionX.g_transparency.setName(cfg_g_transparency_trans[idiom]);
      m_gtPanel.add(m_pjRevolutionX.g_transparency.newInspector(PsPanel.INFO_EXT));
      
      m_atPanel = new Panel();
      m_atPanel.setLayout(new GridLayout(1, 1));
      cfgPanel.add(m_atPanel);
      m_atPanel.removeAll();
      m_pjRevolutionX.a_transparency.setName(cfg_a_transparency_trans[idiom]);
      m_atPanel.add(m_pjRevolutionX.a_transparency.newInspector(PsPanel.INFO_EXT));
      
      m_btPanel = new Panel();
      m_btPanel.setLayout(new GridLayout(1, 1));
      cfgPanel.add(m_btPanel);
      m_btPanel.removeAll();
      m_pjRevolutionX.b_transparency.setName(cfg_b_transparency_trans[idiom]);
      m_btPanel.add(m_pjRevolutionX.b_transparency.newInspector(PsPanel.INFO_EXT));
      
      {
          Label l = new Label(" ");
          l.setFont(new Font("dialog", Font.PLAIN, 2));
          cfgPanel.add(l);
      }
      
      cfgPanel.addLine(1);                 
      
      Label colorLabel = new Label(cfg_color_trans[idiom], Label.CENTER);
      colorLabel.setFont(new Font("dialog", Font.BOLD, 13));
      cfgPanel.add(colorLabel);
      
      {
          Label l = new Label(" ");
          l.setFont(new Font("dialog", Font.PLAIN, 2));
          cfgPanel.add(l);
      }      
                            
      m_pjRevolutionX.f_pc_IP.setTitle(cfg_f_pc_trans[idiom]);            
      cfgPanel.add(m_pjRevolutionX.f_pc_IP);
      m_pjRevolutionX.g_pc_IP.setTitle(cfg_g_pc_trans[idiom]);            
      cfgPanel.add(m_pjRevolutionX.g_pc_IP);
      m_pjRevolutionX.a_pc_IP.setTitle(cfg_a_pc_trans[idiom]);            
      cfgPanel.add(m_pjRevolutionX.a_pc_IP);
      m_pjRevolutionX.b_pc_IP.setTitle(cfg_b_pc_trans[idiom]);            
      cfgPanel.add(m_pjRevolutionX.b_pc_IP);
      
      Label lc = new Label(" ");
      lc.setFont(new Font("dialog", Font.PLAIN, 2));
      cfgPanel.add(lc);                            
      
      tabPanel.addPanel(cfg_panel_trans[idiom], cfgPanel);
      
      // Elements panel
      PsPanel elePanel = new PsPanel();
      
      {
          Label l = new Label(" ");
          l.setFont(new Font("dialog", Font.PLAIN, 8));
          elePanel.add(l);
      }
      
      // elePanel.addLine(1);
      
      Label axesLabel = new Label(ele_axes_trans[idiom], Label.CENTER);
      axesLabel.setFont(new Font("dialog", Font.BOLD, 13));
      elePanel.add(axesLabel);
      
      {
          Label l = new Label(" ");
          l.setFont(new Font("dialog", Font.PLAIN, 2));
          elePanel.add(l);
      }            
      
      PsPanel axesPanel = new PsPanel();
      axesPanel.setLayout(new GridLayout(1, 3));
      
      x_cb = new Checkbox(ele_x_cb_trans[idiom]);
      x_cb.addItemListener(this);
      x_cb.setState(m_pjRevolutionX.x_cb_ini);
      axesPanel.add(x_cb);
      
      y_cb = new Checkbox(ele_y_cb_trans[idiom]);
      y_cb.addItemListener(this);
      y_cb.setState(m_pjRevolutionX.y_cb_ini);
      axesPanel.add(y_cb);
      
      z_cb = new Checkbox(ele_z_cb_trans[idiom]);
      z_cb.addItemListener(this);
      z_cb.setState(m_pjRevolutionX.z_cb_ini);
      axesPanel.add(z_cb);
      
      elePanel.add(axesPanel);

      {
          Label l = new Label(" ");
          l.setFont(new Font("dialog", Font.PLAIN, 2));
          elePanel.add(l);
      }
      
      elePanel.addLine(1);
      
      Label edgesLabel = new Label(ele_edges_trans[idiom], Label.CENTER);
      edgesLabel.setFont(new Font("dialog", Font.BOLD, 13));
      elePanel.add(edgesLabel);
      
      {
          Label l = new Label(" ");
          l.setFont(new Font("dialog", Font.PLAIN, 2));
          elePanel.add(l);
      }                  
         
      PsPanel edgesPanel = new PsPanel();
      edgesPanel.setLayout(new GridLayout(4, 1));
      
      f_cb = new Checkbox(ele_f_cb_trans[idiom]);
      f_cb.addItemListener(this);
      f_cb.setState(m_pjRevolutionX.f_cb_ini);
      edgesPanel.add(f_cb);

      g_cb = new Checkbox(ele_g_cb_trans[idiom]);
      g_cb.addItemListener(this);
      g_cb.setState(m_pjRevolutionX.g_cb_ini);
      edgesPanel.add(g_cb);

      a_cb = new Checkbox(ele_a_cb_trans[idiom]);
      a_cb.addItemListener(this);
      a_cb.setState(m_pjRevolutionX.a_cb_ini);
      edgesPanel.add(a_cb);
      
      b_cb = new Checkbox(ele_b_cb_trans[idiom]);
      b_cb.addItemListener(this);
      b_cb.setState(m_pjRevolutionX.b_cb_ini);
      edgesPanel.add(b_cb);
      
      elePanel.add(edgesPanel);
      
      elePanel.addLine(1);
      
      Label lightingLabel = new Label(ele_lighting_trans[idiom], Label.CENTER);
      lightingLabel.setFont(new Font("dialog", Font.BOLD, 13));
      elePanel.add(lightingLabel);
      
      {
          Label l = new Label(" ");
          l.setFont(new Font("dialog", Font.PLAIN, 2));
          elePanel.add(l);
      }                  
      
      i_cb = new Checkbox(ele_i_cb_trans[idiom]);
      i_cb.addItemListener(this);
      i_cb.setState(m_pjRevolutionX.i_cb_ini);
      elePanel.add(i_cb); 
      
      // elePanel.addLine(1);
      
      tabPanel.addPanel(ele_panel_trans[idiom], elePanel);      
      
      // Validate
      tabPanel.validate();

      super.setParent(parent);
   }; // End of setParent()


   //////////////////////////////////////////////////////////////////////////
   // Update the panel whenever the parent has changed somewhere else
   //////////////////////////////////////////////////////////////////////////
   public boolean update(Object event)
   {
      if (m_pjRevolutionX == event)
      {
         PsPanel.setText(m_fTextField, new String(m_pjRevolutionX.f));
         PsPanel.setText(m_gTextField, new String(m_pjRevolutionX.g));
         PsPanel.setText(m_aTextField, new String(m_pjRevolutionX.a));
         PsPanel.setText(m_bTextField, new String(m_pjRevolutionX.b));         
         
         return(true);
      };
      
      return(super.update(event));
   }; // End of update()


   //////////////////////////////////////////////////////////////////////////
   // Handle action events invoked from text fields
   //////////////////////////////////////////////////////////////////////////
   public void actionPerformed(ActionEvent event)
   {
      if (m_pjRevolutionX == null)
      {
          return;
      };
         
      Object source = event.getSource();

      if ((source == m_fTextField) || (source == m_gTextField) ||
          (source == m_aTextField) || (source == m_bTextField) ||
          (source == updateButton))
      {
    	  int rc = 0;
    	  if ((m_pjRevolutionX.a.compareTo(m_aTextField.getText()) != 0) ||
    		  (m_pjRevolutionX.b.compareTo(m_bTextField.getText()) != 0))
    	  {
    		  m_pjRevolutionX.f = m_fTextField.getText();
    		  m_pjRevolutionX.g = m_gTextField.getText();
    		  m_pjRevolutionX.a = m_aTextField.getText();
    		  m_pjRevolutionX.b = m_bTextField.getText();
    		  
    		  rc = m_pjRevolutionX.updateSurface();
    	  }
    	  else
    	  {
    		  if ((m_pjRevolutionX.f.compareTo(m_fTextField.getText()) != 0) &&
    			  (m_pjRevolutionX.g.compareTo(m_gTextField.getText()) == 0))
    		  {
        		  m_pjRevolutionX.f = m_fTextField.getText();
        		  m_pjRevolutionX.g = m_gTextField.getText();
        		  m_pjRevolutionX.a = m_aTextField.getText();
        		  m_pjRevolutionX.b = m_bTextField.getText();
        		  
    			  rc = m_pjRevolutionX.updateF();
    		  }
    		  else
    		  {
        		  if ((m_pjRevolutionX.f.compareTo(m_fTextField.getText()) == 0) &&
            		  (m_pjRevolutionX.g.compareTo(m_gTextField.getText()) != 0))
            	  {
            		  m_pjRevolutionX.f = m_fTextField.getText();
            		  m_pjRevolutionX.g = m_gTextField.getText();
            		  m_pjRevolutionX.a = m_aTextField.getText();
            		  m_pjRevolutionX.b = m_bTextField.getText();
       			  
          			  rc = m_pjRevolutionX.updateG();
            	  }
        		  else
        		  {
            		  m_pjRevolutionX.f = m_fTextField.getText();
            		  m_pjRevolutionX.g = m_gTextField.getText();
            		  m_pjRevolutionX.a = m_aTextField.getText();
            		  m_pjRevolutionX.b = m_bTextField.getText();
        			  
        			  rc = m_pjRevolutionX.updateSurface();
        		  }
    		  }
    	  }    
    	  
    	  m_pjRevolutionX.updateAxes();
    	  
    	  if (rc == 0)
    	  {
    		  msgLabel.setText(" ");
    		  m_aTextField.setBackground(Color.WHITE);
    		  m_bTextField.setBackground(Color.WHITE);
    		  m_fTextField.setBackground(Color.WHITE);
    		  m_gTextField.setBackground(Color.WHITE);

    		  m_pjRevolutionX.updateAxes();
    		  
    		  m_pjRevolutionX.disp.fit();
    		  
    		  return;
    	  }
    	  
    	  if (rc == 1)
    	  {
    	     msgLabel.setText(rc_invalid_input_trans[idiom]);
    	     m_aTextField.setBackground(errorColor);
    	     return;
    	  }
    	  
    	  if (rc == 2)
    	  {
     	     msgLabel.setText(rc_invalid_input_trans[idiom]);
    	     m_bTextField.setBackground(errorColor);
    	     return;    		  
    	  }    	  
    	  
    	  if (rc == 3)
    	  {
     	     msgLabel.setText(rc_invalid_input_trans[idiom]);
    	     m_fTextField.setBackground(errorColor);
    	     return;    		  
    	  }    	  
    	  
    	  if (rc == 4)
    	  {
        	 msgLabel.setText(rc_f_eval_trans[idiom]);
        	 m_fTextField.setBackground(errorColor);
        	 return;    		      		      		  
    	  }    	  
    	  
    	  if (rc == 5)
    	  {    		  
       	     msgLabel.setText(rc_invalid_input_trans[idiom]);
     	     m_gTextField.setBackground(errorColor);
     	     return;    		      		      		  
    	  }    	  
    	  
    	  if (rc == 6)
    	  {
        	  msgLabel.setText(rc_g_eval_trans[idiom]);
        	  m_gTextField.setBackground(errorColor);
        	  return;    		  
    	  }    	  
      } 
      
      if (source == animateButton)
      {
    	  if (animateButton.getLabel().compareTo(par_animate_trans[idiom]) == 0)
    	  {
    		  animateButton.setLabel(par_stop_trans[idiom]);    		 
    		  animateButton.setBackground(new Color(255, 119, 119));
    	  }
    	  else
    	  {
    		  animateButton.setLabel(par_animate_trans[idiom]);
    		  animateButton.setBackground(new Color(212, 208, 200));
    	  }
    	  
    	  m_pjRevolutionX.animate();
      }
      
   }; // End of actionPerformed()


   //////////////////////////////////////////////////////////////////////////
   // Handle action events invoked from itens
   //////////////////////////////////////////////////////////////////////////
   public void itemStateChanged(ItemEvent event)
   {
       if (m_pjRevolutionX == null)
       {
          return;
       };

       Object source = event.getSource();
       
       if (source == x_cb)
       {
    	   if (x_cb.getState() == false)
    	   {
    	      m_pjRevolutionX.removeGeometry(m_pjRevolutionX.axis3);
    	   }
    	   else
    	   {
    		  m_pjRevolutionX.addGeometry(m_pjRevolutionX.axis3);
    	   }
       }
       
       if (source == y_cb)
       {
    	   if (y_cb.getState() == false)
    	   {
    	      m_pjRevolutionX.removeGeometry(m_pjRevolutionX.axis1);
    	   }
    	   else
    	   {
    		  m_pjRevolutionX.addGeometry(m_pjRevolutionX.axis1);
    	   }
       }
       
       if (source == z_cb)
       {
    	   if (z_cb.getState() == false)
    	   {
    	      m_pjRevolutionX.removeGeometry(m_pjRevolutionX.axis2);
    	   }
    	   else
    	   {
    		  m_pjRevolutionX.addGeometry(m_pjRevolutionX.axis2);
    	   }
       }

       if (source == f_cb)
       {
    	   boolean v = f_cb.getState();
    	   m_pjRevolutionX.f_show_edges = v;
           m_pjRevolutionX.ssrf.showEdges(v);
           m_pjRevolutionX.ssrf.update(m_pjRevolutionX.ssrf);
       }

       if (source == g_cb)
       {
    	   boolean v = g_cb.getState();
    	   m_pjRevolutionX.g_show_edges = v;
           m_pjRevolutionX.ssrg.showEdges(v);
           m_pjRevolutionX.ssrg.update(m_pjRevolutionX.ssrg);
       }
       
       if (source == a_cb)
       {
    	   boolean v = a_cb.getState();
    	   m_pjRevolutionX.a_show_edges = v;
           m_pjRevolutionX.ssra.showEdges(v);
           m_pjRevolutionX.ssra.update(m_pjRevolutionX.ssra);
       }
       
       if (source == b_cb)
       {
    	   boolean v = b_cb.getState();
    	   m_pjRevolutionX.b_show_edges = v;
           m_pjRevolutionX.ssrb.showEdges(v);
           m_pjRevolutionX.ssrb.update(m_pjRevolutionX.ssrb);
       }
       
       if (source == i_cb)
       {
    	   if (i_cb.getState() == false)
    	   {
    	      // m_pjRevolutionX.disp.setLightingModel(PvLightIf.LIGHT_AMBIENT);
    		  m_pjRevolutionX.disp.showDepthcue(false); 
    	      m_pjRevolutionX.disp.update(m_pjRevolutionX.disp);
    	   }
    	   else
    	   {
    		  // m_pjRevolutionX.disp.setLightingModel(PvLightIf.LIGHT_DIRECTION);
    		  m_pjRevolutionX.disp.showDepthcue(true);    		   
    		  m_pjRevolutionX.disp.update(m_pjRevolutionX.disp);
    	   }       
       }
   }; // End of itemStateChanged()
}; // End of PjRevolutionX_IP class
