// SurfaceOfRevoltion class

import jv.geom.*;
import jv.object.PsObject;
import jv.vecmath.*;

public class SurfaceOfRevolution extends PgElementSet
{
    protected PgPolygon m_meridian;
    protected PdVector m_base;
    protected PdVector m_axis;
    protected double m_angle;
    protected double m_angleOffset;
    protected int m_discr;		

    public SurfaceOfRevolution()
    {
        this(3);
        compute();
    }

    public SurfaceOfRevolution(int i)
    {
        super(i);        
        m_base = new PdVector(i);
        m_axis = new PdVector(i);
        
        if (i == 3)
        {
            m_axis.set(0.0D, 0.0D, 1.0D);
        }
        else
        {        	
	        if (i > 3)
	        {
	            m_axis.setConstant(1.0D);
	            m_axis.setEntry(0, 0.0D);
	            m_axis.setEntry(1, 0.0D);
	        }
        }
        
        PgPolygon pgpolygon = new PgPolygon(i);
        pgpolygon.setName("Meridian");
        setMeridian(pgpolygon);
        if (getClass() == (SurfaceOfRevolution.class))
        {
           init();
        }
    }

    public boolean update(Object obj)
    {    	
        if (obj == this)
        {        	
            return m_meridian.update(m_meridian);
        }
        
        if (obj == m_meridian)
        {
           compute();
           return super.update(this);
        }
        
        return super.update(obj);
    }

   
    public void setDiscr(int discr)
    {
    	m_discr = discr;
    }
    
    public void setAngle(double angle)
    {
    	m_angle = angle;
    }
    
    public void setAngleOffset(double angleOffset)
    {
    	m_angleOffset = angleOffset;
    }    
                
    public void updateDiscr(int discr)
    {
    	m_discr = discr;
    	compute();
    	super.update(this);
    }
    
    public void updateAngle(double angle)
    {
    	m_angle = angle;
    	compute();
    	super.update(this);
    }    
    
    public void updateAngleOffset(double angleOffset)
    {
    	m_angleOffset = angleOffset;
    	compute();
    	super.update(this);
    }        

    public boolean setMeridian(PgPolygon pgpolygon)
    {
        if (m_meridian == pgpolygon)
        {
            return true;
        }
        
        if (m_meridian != null)
        {
           m_meridian.removeUpdateListener(this);
        }
        
        m_meridian = pgpolygon;
        
        if (pgpolygon != null)
        {
            pgpolygon.addUpdateListener(this);
        }
        
        return true;
    }

    public void copy(PsObject psobject)
    {
        super.copy(psobject);
        
        if (!(psobject instanceof SurfaceOfRevolution))
        {
            return;
        };
        
        SurfaceOfRevolution ssr = (SurfaceOfRevolution)psobject;
        
        PgPolygon pgpolygon = ssr.getMeridian();
        if (pgpolygon != null)
        {
            m_meridian.copy(pgpolygon);
        }
        else
        {
	        if (m_meridian != null)
	        {
	            m_meridian.removeUpdateListener(this);
	            m_meridian = null;
	        }
        }
        
        m_base.copy(ssr.m_base);
        m_axis.copy(ssr.m_axis);
        m_angle = ssr.m_angle;
        m_angleOffset = ssr.m_angleOffset;
        m_discr = ssr.m_discr;
    }

    public boolean makeVertexTextureFromUV()
    {
        if (super.m_numVertices == 0)
        {
            return true;
        }
        
        if (m_meridian == null)
        {
            return true;
        } 
        else
        {
            int i = m_meridian.getNumVertices();
            int j = m_discr + 1;
            return makeVertexTextureFromUV(i, j, 1);
        }
    }

    public void init()
    {
        super.init();
        
        m_base.setConstant(0.0D);
        m_angle = 2*Math.PI;
        m_angleOffset = 0.0;
        m_discr = 10;
    }

    public Object clone()
    {
        SurfaceOfRevolution ssr = (SurfaceOfRevolution) super.clone();
        
        if (ssr == null)
        {
            return null;
        }
        
        if (m_meridian != null)
        {
            ssr.m_meridian = (PgPolygon)m_meridian.clone();
            ssr.m_meridian.addUpdateListener(ssr);
        }
        
        ssr.m_base = (PdVector)m_base.clone();
        ssr.m_axis = (PdVector)m_axis.clone();
        ssr.m_angle = m_angle;
        ssr.m_angleOffset = m_angleOffset;
        ssr.m_discr = m_discr;
        
        return ssr;
    }

    public boolean compute()
    {
        int i = m_meridian.getNumVertices();
        int j = m_discr + 1;
        setNumVertices(i * j);
        PdMatrix pdmatrix = new PdMatrix(4);
        PuReflect.rotateLine(pdmatrix, m_base, m_axis, m_angleOffset);
        double d = m_angle / ((double)j - 1.0D);
        PdMatrix pdmatrix1 = new PdMatrix(4);
        PuReflect.rotateLine(pdmatrix1, m_base, m_axis, d);
        int k = 0;
        for(int l = 0; l < i; l++)
        {
            PdVector pdvector = m_meridian.getVertex(l);
            super.m_vertex[k].leftMultAffin(pdmatrix, pdvector, true);
            for(int i1 = 1; i1 < j; i1++)
            {
                super.m_vertex[k + 1].leftMultAffin(pdmatrix1, super.m_vertex[k], true);
                k++;
            }

            k++;
        }

        setDimOfElements(4);
        makeQuadrConn(i, j);
        makeQuadrBnd(i, j);
        makeElementNormals();
        makeVertexNormals();
        
        if (hasVertexTextures())
        {        	
            makeVertexTextureFromUV();
        };
        
        return true;
    }
    
    public PgPolygon getMeridian() {return m_meridian;};        
}
